@@ -16,6 +16,28 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import importlib
+import textwrap
+from types import ModuleType
+from typing import Any, Dict, Iterable, NamedTuple, Optional, Tuple, Type
+
+import gitlab
+from gitlab import types as g_types
+from gitlab.exceptions import GitlabParsingError
+
+from .client import Gitlab, GitlabList
+
+__all__ = [
+    "RequiredOptional",
+    "RESTObject",
+    "RESTObjectList",
+    "RESTManager",
+]
+
+
+_URL_ATTRIBUTE_ERROR = (
+    f"https://python-gitlab.readthedocs.io/en/{gitlab.__version__}/"
+    f"faq.html#attribute-error-list"
+)
 
 
 class RESTObject(object):
@@ -29,32 +51,52 @@ class RESTObject(object):
     without ID in the url.
     """
 
-    _id_attr = "id"
+    _id_attr: Optional[str] = "id"
+    _attrs: Dict[str, Any]
+    _created_from_list: bool  # Indicates if object was created from a list() action
+    _module: ModuleType
+    _parent_attrs: Dict[str, Any]
+    _short_print_attr: Optional[str] = None
+    _updated_attrs: Dict[str, Any]
+    manager: "RESTManager"
 
-    def __init__(self, manager, attrs):
+    def __init__(
+        self,
+        manager: "RESTManager",
+        attrs: Dict[str, Any],
+        *,
+        created_from_list: bool = False,
+    ) -> None:
+        if not isinstance(attrs, dict):
+            raise GitlabParsingError(
+                "Attempted to initialize RESTObject with a non-dictionary value: "
+                "{!r}\nThis likely indicates an incorrect or malformed server "
+                "response.".format(attrs)
+            )
         self.__dict__.update(
             {
                 "manager": manager,
                 "_attrs": attrs,
                 "_updated_attrs": {},
                 "_module": importlib.import_module(self.__module__),
+                "_created_from_list": created_from_list,
             }
         )
         self.__dict__["_parent_attrs"] = self.manager.parent_attrs
         self._create_managers()
 
-    def __getstate__(self):
+    def __getstate__(self) -> Dict[str, Any]:
         state = self.__dict__.copy()
         module = state.pop("_module")
         state["_module_name"] = module.__name__
         return state
 
-    def __setstate__(self, state):
+    def __setstate__(self, state: Dict[str, Any]) -> None:
         module_name = state.pop("_module_name")
         self.__dict__.update(state)
         self.__dict__["_module"] = importlib.import_module(module_name)
 
-    def __getattr__(self, name):
+    def __getattr__(self, name: str) -> Any:
         try:
             return self.__dict__["_updated_attrs"][name]
         except KeyError:
@@ -80,67 +122,94 @@ class RESTObject(object):
             except KeyError:
                 try:
                     return self.__dict__["_parent_attrs"][name]
-                except KeyError:
-                    raise AttributeError(name)
+                except KeyError as exc:
+                    message = (
+                        f"{type(self).__name__!r} object has no attribute {name!r}"
+                    )
+                    if self._created_from_list:
+                        message = (
+                            f"{message}\n\n"
+                            + textwrap.fill(
+                                f"{self.__class__!r} was created via a list() call and "
+                                f"only a subset of the data may be present. To ensure "
+                                f"all data is present get the object using a "
+                                f"get(object.id) call. For more details, see:"
+                            )
+                            + f"\n\n{_URL_ATTRIBUTE_ERROR}"
+                        )
+                    raise AttributeError(message) from exc
 
-    def __setattr__(self, name, value):
+    def __setattr__(self, name: str, value: Any) -> None:
         self.__dict__["_updated_attrs"][name] = value
 
-    def __str__(self):
+    def __str__(self) -> str:
         data = self._attrs.copy()
         data.update(self._updated_attrs)
-        return "%s => %s" % (type(self), data)
+        return f"{type(self)} => {data}"
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         if self._id_attr:
-            return "<%s %s:%s>" % (
-                self.__class__.__name__,
-                self._id_attr,
-                self.get_id(),
-            )
+            return f"<{self.__class__.__name__} {self._id_attr}:{self.get_id()}>"
         else:
-            return "<%s>" % self.__class__.__name__
+            return f"<{self.__class__.__name__}>"
 
-    def __eq__(self, other):
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, RESTObject):
+            return NotImplemented
         if self.get_id() and other.get_id():
             return self.get_id() == other.get_id()
         return super(RESTObject, self) == other
 
-    def __ne__(self, other):
+    def __ne__(self, other: object) -> bool:
+        if not isinstance(other, RESTObject):
+            return NotImplemented
         if self.get_id() and other.get_id():
             return self.get_id() != other.get_id()
         return super(RESTObject, self) != other
 
-    def __dir__(self):
-        return super(RESTObject, self).__dir__() + list(self.attributes)
+    def __dir__(self) -> Iterable[str]:
+        return set(self.attributes).union(super(RESTObject, self).__dir__())
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         if not self.get_id():
             return super(RESTObject, self).__hash__()
         return hash(self.get_id())
 
-    def _create_managers(self):
-        managers = getattr(self, "_managers", None)
-        if managers is None:
-            return
-
-        for attr, cls_name in self._managers:
+    def _create_managers(self) -> None:
+        # NOTE(jlvillal): We are creating our managers by looking at the class
+        # annotations. If an attribute is annotated as being a *Manager type
+        # then we create the manager and assign it to the attribute.
+        for attr, annotation in sorted(self.__annotations__.items()):
+            # We ignore creating a manager for the 'manager' attribute as that
+            # is done in the self.__init__() method
+            if attr in ("manager",):
+                continue
+            if not isinstance(annotation, (type, str)):
+                continue
+            if isinstance(annotation, type):
+                cls_name = annotation.__name__
+            else:
+                cls_name = annotation
+            # All *Manager classes are used except for the base "RESTManager" class
+            if cls_name == "RESTManager" or not cls_name.endswith("Manager"):
+                continue
             cls = getattr(self._module, cls_name)
             manager = cls(self.manager.gitlab, parent=self)
+            # Since we have our own __setattr__ method, we can't use setattr()
             self.__dict__[attr] = manager
 
-    def _update_attrs(self, new_attrs):
+    def _update_attrs(self, new_attrs: Dict[str, Any]) -> None:
         self.__dict__["_updated_attrs"] = {}
-        self.__dict__["_attrs"].update(new_attrs)
+        self.__dict__["_attrs"] = new_attrs
 
-    def get_id(self):
+    def get_id(self) -> Any:
         """Returns the id of the resource."""
         if self._id_attr is None or not hasattr(self, self._id_attr):
             return None
         return getattr(self, self._id_attr)
 
     @property
-    def attributes(self):
+    def attributes(self) -> Dict[str, Any]:
         d = self.__dict__["_updated_attrs"].copy()
         d.update(self.__dict__["_attrs"])
         d.update(self.__dict__["_parent_attrs"])
@@ -153,7 +222,7 @@ class RESTObjectList(object):
     This generator uses the Gitlab pagination system to fetch new data when
     required.
 
-    Note: you should not instanciate such objects, they are returned by calls
+    Note: you should not instantiate such objects, they are returned by calls
     to RESTManager.list()
 
     Args:
@@ -162,7 +231,9 @@ class RESTObjectList(object):
         _list: A GitlabList object
     """
 
-    def __init__(self, manager, obj_cls, _list):
+    def __init__(
+        self, manager: "RESTManager", obj_cls: Type[RESTObject], _list: GitlabList
+    ) -> None:
         """Creates an objects list from a GitlabList.
 
         You should not create objects of this type, but use managers list()
@@ -177,26 +248,26 @@ class RESTObjectList(object):
         self._obj_cls = obj_cls
         self._list = _list
 
-    def __iter__(self):
+    def __iter__(self) -> "RESTObjectList":
         return self
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self._list)
 
-    def __next__(self):
+    def __next__(self) -> RESTObject:
         return self.next()
 
-    def next(self):
+    def next(self) -> RESTObject:
         data = self._list.next()
-        return self._obj_cls(self.manager, data)
+        return self._obj_cls(self.manager, data, created_from_list=True)
 
     @property
-    def current_page(self):
+    def current_page(self) -> int:
         """The current page number."""
         return self._list.current_page
 
     @property
-    def prev_page(self):
+    def prev_page(self) -> Optional[int]:
         """The previous page number.
 
         If None, the current page is the first.
@@ -204,7 +275,7 @@ class RESTObjectList(object):
         return self._list.prev_page
 
     @property
-    def next_page(self):
+    def next_page(self) -> Optional[int]:
         """The next page number.
 
         If None, the current page is the last.
@@ -212,21 +283,26 @@ class RESTObjectList(object):
         return self._list.next_page
 
     @property
-    def per_page(self):
+    def per_page(self) -> int:
         """The number of items per page."""
         return self._list.per_page
 
     @property
-    def total_pages(self):
+    def total_pages(self) -> Optional[int]:
         """The total number of pages."""
         return self._list.total_pages
 
     @property
-    def total(self):
+    def total(self) -> Optional[int]:
         """The total number of items."""
         return self._list.total
 
 
+class RequiredOptional(NamedTuple):
+    required: Tuple[str, ...] = tuple()
+    optional: Tuple[str, ...] = tuple()
+
+
 class RESTManager(object):
     """Base class for CRUD operations on objects.
 
@@ -236,15 +312,23 @@ class RESTManager(object):
     ``_obj_cls``: The class of objects that will be created
     """
 
-    _path = None
-    _obj_cls = None
+    _create_attrs: RequiredOptional = RequiredOptional()
+    _update_attrs: RequiredOptional = RequiredOptional()
+    _path: Optional[str] = None
+    _obj_cls: Optional[Type[RESTObject]] = None
+    _from_parent_attrs: Dict[str, Any] = {}
+    _types: Dict[str, Type[g_types.GitlabAttribute]] = {}
 
-    def __init__(self, gl, parent=None):
+    _computed_path: Optional[str]
+    _parent: Optional[RESTObject]
+    _parent_attrs: Dict[str, Any]
+    gitlab: Gitlab
+
+    def __init__(self, gl: Gitlab, parent: Optional[RESTObject] = None) -> None:
         """REST manager constructor.
 
         Args:
-            gl (Gitlab): :class:`~gitlab.Gitlab` connection to use to make
-                         requests.
+            gl: :class:`~gitlab.Gitlab` connection to use to make requests.
             parent: REST object to which the manager is attached.
         """
         self.gitlab = gl
@@ -252,14 +336,16 @@ class RESTManager(object):
         self._computed_path = self._compute_path()
 
     @property
-    def parent_attrs(self):
+    def parent_attrs(self) -> Optional[Dict[str, Any]]:
         return self._parent_attrs
 
-    def _compute_path(self, path=None):
+    def _compute_path(self, path: Optional[str] = None) -> Optional[str]:
         self._parent_attrs = {}
         if path is None:
             path = self._path
-        if self._parent is None or not hasattr(self, "_from_parent_attrs"):
+        if path is None:
+            return None
+        if self._parent is None or not self._from_parent_attrs:
             return path
 
         data = {
@@ -267,8 +353,8 @@ class RESTManager(object):
             for self_attr, parent_attr in self._from_parent_attrs.items()
         }
         self._parent_attrs = data
-        return path % data
+        return path.format(**data)
 
     @property
-    def path(self):
+    def path(self) -> Optional[str]:
         return self._computed_path
