@@ -16,6 +16,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Wrapper for the GitLab API."""
 
+import os
 import time
 from typing import Any, cast, Dict, List, Optional, Tuple, TYPE_CHECKING, Union
 
@@ -29,8 +30,9 @@ import gitlab.exceptions
 from gitlab import utils
 
 REDIRECT_MSG = (
-    "python-gitlab detected an http to https redirection. You "
-    "must update your GitLab URL to use https:// to avoid issues."
+    "python-gitlab detected a {status_code} ({reason!r}) redirection. You must update "
+    "your GitLab URL to the correct URL to avoid issues. The redirection was from: "
+    "{source!r} to {target!r}"
 )
 
 
@@ -38,25 +40,27 @@ class Gitlab(object):
     """Represents a GitLab server connection.
 
     Args:
-        url (str): The URL of the GitLab server.
-        private_token (str): The user private token
-        oauth_token (str): An oauth token
-        job_token (str): A CI job token
-        ssl_verify (bool|str): Whether SSL certificates should be validated. If
+        url: The URL of the GitLab server (defaults to https://gitlab.com).
+        private_token: The user private token
+        oauth_token: An oauth token
+        job_token: A CI job token
+        ssl_verify: Whether SSL certificates should be validated. If
             the value is a string, it is the path to a CA file used for
             certificate validation.
-        timeout (float): Timeout to use for requests to the GitLab server.
-        http_username (str): Username for HTTP authentication
-        http_password (str): Password for HTTP authentication
-        api_version (str): Gitlab API version to use (support for 4 only)
-        pagination (str): Can be set to 'keyset' to use keyset pagination
-        order_by (str): Set order_by globally
-        user_agent (str): A custom user agent to use for making HTTP requests.
+        timeout: Timeout to use for requests to the GitLab server.
+        http_username: Username for HTTP authentication
+        http_password: Password for HTTP authentication
+        api_version: Gitlab API version to use (support for 4 only)
+        pagination: Can be set to 'keyset' to use keyset pagination
+        order_by: Set order_by globally
+        user_agent: A custom user agent to use for making HTTP requests.
+        retry_transient_errors: Whether to retry after 500, 502, 503, or
+            504 responses. Defaults to False.
     """
 
     def __init__(
         self,
-        url: str,
+        url: Optional[str] = None,
         private_token: Optional[str] = None,
         oauth_token: Optional[str] = None,
         job_token: Optional[str] = None,
@@ -70,15 +74,17 @@ class Gitlab(object):
         pagination: Optional[str] = None,
         order_by: Optional[str] = None,
         user_agent: str = gitlab.const.USER_AGENT,
+        retry_transient_errors: bool = False,
     ) -> None:
 
         self._api_version = str(api_version)
         self._server_version: Optional[str] = None
         self._server_revision: Optional[str] = None
-        self._base_url = url.rstrip("/")
-        self._url = "%s/api/v%s" % (self._base_url, api_version)
+        self._base_url = self._get_base_url(url)
+        self._url = f"{self._base_url}/api/v{api_version}"
         #: Timeout to use for requests to gitlab server
         self.timeout = timeout
+        self.retry_transient_errors = retry_transient_errors
         #: Headers that will be used in request to GitLab
         self.headers = {"User-Agent": user_agent}
 
@@ -101,7 +107,7 @@ class Gitlab(object):
 
         # We only support v4 API at this time
         if self._api_version not in ("4",):
-            raise ModuleNotFoundError(name="gitlab.v%s.objects" % self._api_version)
+            raise ModuleNotFoundError(name=f"gitlab.v{self._api_version}.objects")
         # NOTE: We must delay import of gitlab.v4.objects until now or
         # otherwise it will cause circular import errors
         import gitlab.v4.objects
@@ -175,6 +181,8 @@ class Gitlab(object):
         """See :class:`~gitlab.v4.objects.VariableManager`"""
         self.personal_access_tokens = objects.PersonalAccessTokenManager(self)
         """See :class:`~gitlab.v4.objects.PersonalAccessTokenManager`"""
+        self.topics = objects.TopicManager(self)
+        """See :class:`~gitlab.v4.objects.TopicManager`"""
 
     def __enter__(self) -> "Gitlab":
         return self
@@ -191,7 +199,7 @@ class Gitlab(object):
         self.__dict__.update(state)
         # We only support v4 API at this time
         if self._api_version not in ("4",):
-            raise ModuleNotFoundError(name="gitlab.v%s.objects" % self._api_version)
+            raise ModuleNotFoundError(name=f"gitlab.v{self._api_version}.objects")
         # NOTE: We must delay import of gitlab.v4.objects until now or
         # otherwise it will cause circular import errors
         import gitlab.v4.objects
@@ -220,11 +228,11 @@ class Gitlab(object):
         """Create a Gitlab connection from configuration files.
 
         Args:
-            gitlab_id (str): ID of the configuration section.
+            gitlab_id: ID of the configuration section.
             config_files list[str]: List of paths to configuration files.
 
         Returns:
-            (gitlab.Gitlab): A Gitlab connection.
+            A Gitlab connection.
 
         Raises:
             gitlab.config.GitlabDataError: If the configuration is not correct.
@@ -246,8 +254,90 @@ class Gitlab(object):
             pagination=config.pagination,
             order_by=config.order_by,
             user_agent=config.user_agent,
+            retry_transient_errors=config.retry_transient_errors,
         )
 
+    @classmethod
+    def merge_config(
+        cls,
+        options: dict,
+        gitlab_id: Optional[str] = None,
+        config_files: Optional[List[str]] = None,
+    ) -> "Gitlab":
+        """Create a Gitlab connection by merging configuration with
+        the following precedence:
+
+        1. Explicitly provided CLI arguments,
+        2. Environment variables,
+        3. Configuration files:
+            a. explicitly defined config files:
+                i. via the `--config-file` CLI argument,
+                ii. via the `PYTHON_GITLAB_CFG` environment variable,
+            b. user-specific config file,
+            c. system-level config file,
+        4. Environment variables always present in CI (CI_SERVER_URL, CI_JOB_TOKEN).
+
+        Args:
+            options: A dictionary of explicitly provided key-value options.
+            gitlab_id: ID of the configuration section.
+            config_files: List of paths to configuration files.
+        Returns:
+            (gitlab.Gitlab): A Gitlab connection.
+
+        Raises:
+            gitlab.config.GitlabDataError: If the configuration is not correct.
+        """
+        config = gitlab.config.GitlabConfigParser(
+            gitlab_id=gitlab_id, config_files=config_files
+        )
+        url = (
+            options.get("server_url")
+            or config.url
+            or os.getenv("CI_SERVER_URL")
+            or gitlab.const.DEFAULT_URL
+        )
+        private_token, oauth_token, job_token = cls._merge_auth(options, config)
+
+        return cls(
+            url=url,
+            private_token=private_token,
+            oauth_token=oauth_token,
+            job_token=job_token,
+            ssl_verify=options.get("ssl_verify") or config.ssl_verify,
+            timeout=options.get("timeout") or config.timeout,
+            api_version=options.get("api_version") or config.api_version,
+            per_page=options.get("per_page") or config.per_page,
+            pagination=options.get("pagination") or config.pagination,
+            order_by=options.get("order_by") or config.order_by,
+            user_agent=options.get("user_agent") or config.user_agent,
+        )
+
+    @staticmethod
+    def _merge_auth(options: dict, config: gitlab.config.GitlabConfigParser) -> Tuple:
+        """
+        Return a tuple where at most one of 3 token types ever has a value.
+        Since multiple types of tokens may be present in the environment,
+        options, or config files, this precedence ensures we don't
+        inadvertently cause errors when initializing the client.
+
+        This is especially relevant when executed in CI where user and
+        CI-provided values are both available.
+        """
+        private_token = options.get("private_token") or config.private_token
+        oauth_token = options.get("oauth_token") or config.oauth_token
+        job_token = (
+            options.get("job_token") or config.job_token or os.getenv("CI_JOB_TOKEN")
+        )
+
+        if private_token:
+            return (private_token, None, None)
+        if oauth_token:
+            return (None, oauth_token, None)
+        if job_token:
+            return (None, None, job_token)
+
+        return (None, None, None)
+
     def auth(self) -> None:
         """Performs an authentication using private token.
 
@@ -263,9 +353,8 @@ class Gitlab(object):
         object.
 
         Returns:
-            tuple (str, str): The server version and server revision.
-                              ('unknown', 'unknwown') if the server doesn't
-                              perform as expected.
+            The server version and server revision.
+                ('unknown', 'unknown') if the server doesn't perform as expected.
         """
         if self._server_version is None:
             try:
@@ -287,7 +376,7 @@ class Gitlab(object):
         """Validate a gitlab CI configuration.
 
         Args:
-            content (txt): The .gitlab-ci.yml content
+            content: The .gitlab-ci.yml content
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -295,8 +384,7 @@ class Gitlab(object):
             GitlabVerifyError: If the validation could not be done
 
         Returns:
-            tuple: (True, []) if the file is valid, (False, errors(list))
-                otherwise
+            (True, []) if the file is valid, (False, errors(list)) otherwise
         """
         post_data = {"content": content}
         data = self.http_post("/ci/lint", post_data=post_data, **kwargs)
@@ -311,11 +399,9 @@ class Gitlab(object):
         """Render an arbitrary Markdown document.
 
         Args:
-            text (str): The markdown text to render
-            gfm (bool): Render text using GitLab Flavored Markdown. Default is
-                False
-            project (str): Full path of a project used a context when `gfm` is
-                True
+            text: The markdown text to render
+            gfm: Render text using GitLab Flavored Markdown. Default is False
+            project: Full path of a project used a context when `gfm` is True
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -323,7 +409,7 @@ class Gitlab(object):
             GitlabMarkdownError: If the server cannot perform the request
 
         Returns:
-            str: The HTML rendering of the markdown text.
+            The HTML rendering of the markdown text.
         """
         post_data = {"text": text, "gfm": gfm}
         if project is not None:
@@ -345,7 +431,7 @@ class Gitlab(object):
             GitlabGetError: If the server cannot perform the request
 
         Returns:
-            dict: The current license information
+            The current license information
         """
         result = self.http_get("/license", **kwargs)
         if isinstance(result, dict):
@@ -357,7 +443,7 @@ class Gitlab(object):
         """Add a new license.
 
         Args:
-            license (str): The license string
+            license: The license string
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -365,7 +451,7 @@ class Gitlab(object):
             GitlabPostError: If the server cannot perform the request
 
         Returns:
-            dict: The new license information
+            The new license information
         """
         data = {"license": license}
         result = self.http_post("/license", post_data=data, **kwargs)
@@ -403,7 +489,7 @@ class Gitlab(object):
             self.headers.pop("JOB-TOKEN", None)
 
         if self.oauth_token:
-            self.headers["Authorization"] = "Bearer %s" % self.oauth_token
+            self.headers["Authorization"] = f"Bearer {self.oauth_token}"
             self.headers.pop("PRIVATE-TOKEN", None)
             self.headers.pop("JOB-TOKEN", None)
 
@@ -421,7 +507,7 @@ class Gitlab(object):
         import logging
         from http.client import HTTPConnection  # noqa
 
-        HTTPConnection.debuglevel = 1  # type: ignore
+        HTTPConnection.debuglevel = 1
         logging.basicConfig()
         logging.getLogger().setLevel(logging.DEBUG)
         requests_log = logging.getLogger("requests.packages.urllib3")
@@ -436,6 +522,17 @@ class Gitlab(object):
             "verify": self.ssl_verify,
         }
 
+    def _get_base_url(self, url: Optional[str] = None) -> str:
+        """Return the base URL with the trailing slash stripped.
+        If the URL is a Falsy value, return the default URL.
+        Returns:
+            The base URL
+        """
+        if not url:
+            return gitlab.const.DEFAULT_URL
+
+        return url.rstrip("/")
+
     def _build_url(self, path: str) -> str:
         """Returns the full url from path.
 
@@ -443,40 +540,45 @@ class Gitlab(object):
         it to the stored url.
 
         Returns:
-            str: The full URL
+            The full URL
         """
         if path.startswith("http://") or path.startswith("https://"):
             return path
         else:
-            return "%s%s" % (self._url, path)
+            return f"{self._url}{path}"
 
     def _check_redirects(self, result: requests.Response) -> None:
-        # Check the requests history to detect http to https redirections.
-        # If the initial verb is POST, the next request will use a GET request,
-        # leading to an unwanted behaviour.
-        # If the initial verb is PUT, the data will not be send with the next
-        # request.
-        # If we detect a redirection to https with a POST or a PUT request, we
+        # Check the requests history to detect 301/302 redirections.
+        # If the initial verb is POST or PUT, the redirected request will use a
+        # GET request, leading to unwanted behaviour.
+        # If we detect a redirection with a POST or a PUT request, we
         # raise an exception with a useful error message.
-        if result.history and self._base_url.startswith("http:"):
-            for item in result.history:
-                if item.status_code not in (301, 302):
-                    continue
-                # GET methods can be redirected without issue
-                if item.request.method == "GET":
-                    continue
-                # Did we end-up with an https:// URL?
-                location = item.headers.get("Location", None)
-                if location and location.startswith("https://"):
-                    raise gitlab.exceptions.RedirectError(REDIRECT_MSG)
+        if not result.history:
+            return
+
+        for item in result.history:
+            if item.status_code not in (301, 302):
+                continue
+            # GET methods can be redirected without issue
+            if item.request.method == "GET":
+                continue
+            target = item.headers.get("location")
+            raise gitlab.exceptions.RedirectError(
+                REDIRECT_MSG.format(
+                    status_code=item.status_code,
+                    reason=item.reason,
+                    source=item.url,
+                    target=target,
+                )
+            )
 
     def _prepare_send_data(
         self,
         files: Optional[Dict[str, Any]] = None,
-        post_data: Optional[Dict[str, Any]] = None,
+        post_data: Optional[Union[Dict[str, Any], bytes]] = None,
         raw: bool = False,
     ) -> Tuple[
-        Optional[Dict[str, Any]],
+        Optional[Union[Dict[str, Any], bytes]],
         Optional[Union[Dict[str, Any], MultipartEncoder]],
         str,
     ]:
@@ -486,6 +588,8 @@ class Gitlab(object):
             else:
                 # booleans does not exists for data (neither for MultipartEncoder):
                 # cast to string int to avoid: 'bool' object has no attribute 'encode'
+                if TYPE_CHECKING:
+                    assert isinstance(post_data, dict)
                 for k, v in post_data.items():
                     if isinstance(v, bool):
                         post_data[k] = str(int(v))
@@ -505,36 +609,31 @@ class Gitlab(object):
         verb: str,
         path: str,
         query_data: Optional[Dict[str, Any]] = None,
-        post_data: Optional[Dict[str, Any]] = None,
+        post_data: Optional[Union[Dict[str, Any], bytes]] = None,
         raw: bool = False,
         streamed: bool = False,
         files: Optional[Dict[str, Any]] = None,
         timeout: Optional[float] = None,
         obey_rate_limit: bool = True,
-        retry_transient_errors: bool = False,
         max_retries: int = 10,
         **kwargs: Any,
     ) -> requests.Response:
         """Make an HTTP request to the Gitlab server.
 
         Args:
-            verb (str): The HTTP method to call ('get', 'post', 'put',
-                        'delete')
-            path (str): Path or full URL to query ('/projects' or
+            verb: The HTTP method to call ('get', 'post', 'put', 'delete')
+            path: Path or full URL to query ('/projects' or
                         'http://whatever/v4/api/projecs')
-            query_data (dict): Data to send as query parameters
-            post_data (dict): Data to send in the body (will be converted to
+            query_data: Data to send as query parameters
+            post_data: Data to send in the body (will be converted to
                               json by default)
-            raw (bool): If True, do not convert post_data to json
-            streamed (bool): Whether the data should be streamed
-            files (dict): The files to send to the server
-            timeout (float): The timeout, in seconds, for the request
-            obey_rate_limit (bool): Whether to obey 429 Too Many Request
+            raw: If True, do not convert post_data to json
+            streamed: Whether the data should be streamed
+            files: The files to send to the server
+            timeout: The timeout, in seconds, for the request
+            obey_rate_limit: Whether to obey 429 Too Many Request
                                     responses. Defaults to True.
-            retry_transient_errors (bool): Whether to retry after 500, 502,
-                                           503, or 504 responses. Defaults
-                                           to False.
-            max_retries (int): Max retries after 429 or transient errors,
+            max_retries: Max retries after 429 or transient errors,
                                set to -1 to retry forever. Defaults to 10.
             **kwargs: Extra options to send to the server (e.g. sudo)
 
@@ -576,28 +675,28 @@ class Gitlab(object):
         json, data, content_type = self._prepare_send_data(files, post_data, raw)
         opts["headers"]["Content-type"] = content_type
 
-        # Requests assumes that `.` should not be encoded as %2E and will make
-        # changes to urls using this encoding. Using a prepped request we can
-        # get the desired behavior.
-        # The Requests behavior is right but it seems that web servers don't
-        # always agree with this decision (this is the case with a default
-        # gitlab installation)
-        req = requests.Request(verb, url, json=json, data=data, params=params, **opts)
-        prepped = self.session.prepare_request(req)
-        prepped.url = utils.sanitized_url(prepped.url)
-        settings = self.session.merge_environment_settings(
-            prepped.url, {}, streamed, verify, None
-        )
-
         cur_retries = 0
         while True:
-            result = self.session.send(prepped, timeout=timeout, **settings)
+            result = self.session.request(
+                method=verb,
+                url=url,
+                json=json,
+                data=data,
+                params=params,
+                timeout=timeout,
+                verify=verify,
+                stream=streamed,
+                **opts,
+            )
 
             self._check_redirects(result)
 
             if 200 <= result.status_code < 300:
                 return result
 
+            retry_transient_errors = kwargs.get(
+                "retry_transient_errors", self.retry_transient_errors
+            )
             if (429 == result.status_code and obey_rate_limit) or (
                 result.status_code in [500, 502, 503, 504] and retry_transient_errors
             ):
@@ -642,11 +741,11 @@ class Gitlab(object):
         """Make a GET request to the Gitlab server.
 
         Args:
-            path (str): Path or full URL to query ('/projects' or
+            path: Path or full URL to query ('/projects' or
                         'http://whatever/v4/api/projecs')
-            query_data (dict): Data to send as query parameters
-            streamed (bool): Whether the data should be streamed
-            raw (bool): If True do not try to parse the output as json
+            query_data: Data to send as query parameters
+            streamed: Whether the data should be streamed
+            raw: If True do not try to parse the output as json
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
@@ -687,14 +786,14 @@ class Gitlab(object):
         """Make a GET request to the Gitlab server for list-oriented queries.
 
         Args:
-            path (str): Path or full URL to query ('/projects' or
+            path: Path or full URL to query ('/projects' or
                         'http://whatever/v4/api/projects')
-            query_data (dict): Data to send as query parameters
+            query_data: Data to send as query parameters
             **kwargs: Extra options to send to the server (e.g. sudo, page,
                       per_page)
 
         Returns:
-            list: A list of the objects returned by the server. If `as_list` is
+            A list of the objects returned by the server. If `as_list` is
             False and no pagination-related arguments (`page`, `per_page`,
             `all`) are defined then a GitlabList object (generator) is returned
             instead. This object will make API calls when needed to fetch the
@@ -736,13 +835,13 @@ class Gitlab(object):
         """Make a POST request to the Gitlab server.
 
         Args:
-            path (str): Path or full URL to query ('/projects' or
+            path: Path or full URL to query ('/projects' or
                         'http://whatever/v4/api/projecs')
-            query_data (dict): Data to send as query parameters
-            post_data (dict): Data to send in the body (will be converted to
+            query_data: Data to send as query parameters
+            post_data: Data to send in the body (will be converted to
                               json by default)
-            raw (bool): If True, do not convert post_data to json
-            files (dict): The files to send to the server
+            raw: If True, do not convert post_data to json
+            files: The files to send to the server
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
@@ -777,7 +876,7 @@ class Gitlab(object):
         self,
         path: str,
         query_data: Optional[Dict[str, Any]] = None,
-        post_data: Optional[Dict[str, Any]] = None,
+        post_data: Optional[Union[Dict[str, Any], bytes]] = None,
         raw: bool = False,
         files: Optional[Dict[str, Any]] = None,
         **kwargs: Any,
@@ -785,13 +884,13 @@ class Gitlab(object):
         """Make a PUT request to the Gitlab server.
 
         Args:
-            path (str): Path or full URL to query ('/projects' or
+            path: Path or full URL to query ('/projects' or
                         'http://whatever/v4/api/projecs')
-            query_data (dict): Data to send as query parameters
-            post_data (dict): Data to send in the body (will be converted to
+            query_data: Data to send as query parameters
+            post_data: Data to send in the body (will be converted to
                               json by default)
-            raw (bool): If True, do not convert post_data to json
-            files (dict): The files to send to the server
+            raw: If True, do not convert post_data to json
+            files: The files to send to the server
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
@@ -824,7 +923,7 @@ class Gitlab(object):
         """Make a DELETE request to the Gitlab server.
 
         Args:
-            path (str): Path or full URL to query ('/projects' or
+            path: Path or full URL to query ('/projects' or
                         'http://whatever/v4/api/projecs')
             **kwargs: Extra options to send to the server (e.g. sudo)
 
@@ -843,8 +942,8 @@ class Gitlab(object):
         """Search GitLab resources matching the provided string.'
 
         Args:
-            scope (str): Scope of the search
-            search (str): Search string
+            scope: Scope of the search
+            search: Search string
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -852,7 +951,7 @@ class Gitlab(object):
             GitlabSearchError: If the server failed to perform the request
 
         Returns:
-            GitlabList: A list of dicts describing the resources found.
+            A list of dicts describing the resources found.
         """
         data = {"scope": scope, "search": search}
         return self.http_list("/search", query_data=data, **kwargs)
@@ -900,14 +999,12 @@ class GitlabList(object):
             self._next_url = next_url
         except KeyError:
             self._next_url = None
-        self._current_page: Optional[Union[str, int]] = result.headers.get("X-Page")
-        self._prev_page: Optional[Union[str, int]] = result.headers.get("X-Prev-Page")
-        self._next_page: Optional[Union[str, int]] = result.headers.get("X-Next-Page")
-        self._per_page: Optional[Union[str, int]] = result.headers.get("X-Per-Page")
-        self._total_pages: Optional[Union[str, int]] = result.headers.get(
-            "X-Total-Pages"
-        )
-        self._total: Optional[Union[str, int]] = result.headers.get("X-Total")
+        self._current_page: Optional[str] = result.headers.get("X-Page")
+        self._prev_page: Optional[str] = result.headers.get("X-Prev-Page")
+        self._next_page: Optional[str] = result.headers.get("X-Next-Page")
+        self._per_page: Optional[str] = result.headers.get("X-Per-Page")
+        self._total_pages: Optional[str] = result.headers.get("X-Total-Pages")
+        self._total: Optional[str] = result.headers.get("X-Total")
 
         try:
             self._data: List[Dict[str, Any]] = result.json()
@@ -948,19 +1045,22 @@ class GitlabList(object):
             assert self._per_page is not None
         return int(self._per_page)
 
+    # NOTE(jlvillal): When a query returns more than 10,000 items, GitLab doesn't return
+    # the headers 'x-total-pages' and 'x-total'. In those cases we return None.
+    # https://docs.gitlab.com/ee/user/gitlab_com/index.html#pagination-response-headers
     @property
-    def total_pages(self) -> int:
+    def total_pages(self) -> Optional[int]:
         """The total number of pages."""
-        if TYPE_CHECKING:
-            assert self._total_pages is not None
-        return int(self._total_pages)
+        if self._total_pages is not None:
+            return int(self._total_pages)
+        return None
 
     @property
-    def total(self) -> int:
+    def total(self) -> Optional[int]:
         """The total number of items."""
-        if TYPE_CHECKING:
-            assert self._total is not None
-        return int(self._total)
+        if self._total is not None:
+            return int(self._total)
+        return None
 
     def __iter__(self) -> "GitlabList":
         return self
