@@ -46,9 +46,9 @@ class GitlabError(Exception):
 
     def __str__(self) -> str:
         if self.response_code is not None:
-            return "{0}: {1}".format(self.response_code, self.error_message)
+            return f"{self.response_code}: {self.error_message}"
         else:
-            return "{0}".format(self.error_message)
+            return f"{self.error_message}"
 
 
 class GitlabAuthenticationError(GitlabError):
@@ -111,6 +111,10 @@ class GitlabProjectDeployKeyError(GitlabOperationError):
     pass
 
 
+class GitlabPromoteError(GitlabOperationError):
+    pass
+
+
 class GitlabCancelError(GitlabOperationError):
     pass
 
@@ -293,8 +297,7 @@ def on_http_error(error: Type[Exception]) -> Callable[[__F], __F]:
     raise specialized exceptions instead.
 
     Args:
-        error(Exception): The exception type to raise -- must inherit from
-            GitlabError
+        The exception type to raise -- must inherit from GitlabError
     """
 
     def wrap(f: __F) -> __F:
