@@ -15,7 +15,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import warnings
 from types import ModuleType
 from typing import (
     Any,
@@ -87,14 +86,14 @@ class GetMixin(_RestManagerBase):
         """Retrieve a single object.
 
         Args:
-            id (int or str): ID of the object to retrieve
-            lazy (bool): If True, don't request the server, but create a
+            id: ID of the object to retrieve
+            lazy: If True, don't request the server, but create a
                          shallow object giving access to the managers. This is
                          useful if you want to avoid useless calls to the API.
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
-            object: The generated RESTObject.
+            The generated RESTObject.
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
@@ -102,7 +101,7 @@ class GetMixin(_RestManagerBase):
         """
         if not isinstance(id, int):
             id = utils.clean_str_id(id)
-        path = "%s/%s" % (self.path, id)
+        path = f"{self.path}/{id}"
         if TYPE_CHECKING:
             assert self._obj_cls is not None
         if lazy is True:
@@ -135,7 +134,7 @@ class GetWithoutIdMixin(_RestManagerBase):
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
-            object: The generated RESTObject
+            The generated RESTObject
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
@@ -174,7 +173,7 @@ class RefreshMixin(_RestObjectBase):
             GitlabGetError: If the server cannot perform the request
         """
         if self._id_attr:
-            path = "%s/%s" % (self.manager.path, self.id)
+            path = f"{self.manager.path}/{self.id}"
         else:
             if TYPE_CHECKING:
                 assert self.manager.path is not None
@@ -200,15 +199,15 @@ class ListMixin(_RestManagerBase):
         """Retrieve a list of objects.
 
         Args:
-            all (bool): If True, return all the items, without pagination
-            per_page (int): Number of items to retrieve per request
-            page (int): ID of the page to return (starts with page 1)
-            as_list (bool): If set to False and no pagination option is
+            all: If True, return all the items, without pagination
+            per_page: Number of items to retrieve per request
+            page: ID of the page to return (starts with page 1)
+            as_list: If set to False and no pagination option is
                 defined, return a generator instead of a list
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
-            list: The list of objects, or a generator if `as_list` is False
+            The list of objects, or a generator if `as_list` is False
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
@@ -241,7 +240,7 @@ class ListMixin(_RestManagerBase):
             assert self._obj_cls is not None
         obj = self.gitlab.http_list(path, **data)
         if isinstance(obj, list):
-            return [self._obj_cls(self, item) for item in obj]
+            return [self._obj_cls(self, item, created_from_list=True) for item in obj]
         else:
             return base.RESTObjectList(self, self._obj_cls, obj)
 
@@ -274,7 +273,7 @@ class CreateMixin(_RestManagerBase):
                 missing.append(attr)
                 continue
         if missing:
-            raise AttributeError("Missing attributes: %s" % ", ".join(missing))
+            raise AttributeError(f"Missing attributes: {', '.join(missing)}")
 
     @exc.on_http_error(exc.GitlabCreateError)
     def create(
@@ -283,12 +282,12 @@ class CreateMixin(_RestManagerBase):
         """Create a new object.
 
         Args:
-            data (dict): parameters to send to the server to create the
+            data: parameters to send to the server to create the
                          resource
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
-            RESTObject: a new instance of the managed object class built with
+            A new instance of the managed object class built with
                 the data sent by the server
 
         Raises:
@@ -350,7 +349,7 @@ class UpdateMixin(_RestManagerBase):
                 missing.append(attr)
                 continue
         if missing:
-            raise AttributeError("Missing attributes: %s" % ", ".join(missing))
+            raise AttributeError(f"Missing attributes: {', '.join(missing)}")
 
     def _get_update_method(
         self,
@@ -358,7 +357,7 @@ class UpdateMixin(_RestManagerBase):
         """Return the HTTP method to use.
 
         Returns:
-            object: http_put (default) or http_post
+            http_put (default) or http_post
         """
         if self._update_uses_post:
             http_method = self.gitlab.http_post
@@ -371,7 +370,7 @@ class UpdateMixin(_RestManagerBase):
         self,
         id: Optional[Union[str, int]] = None,
         new_data: Optional[Dict[str, Any]] = None,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> Dict[str, Any]:
         """Update an object on the server.
 
@@ -381,7 +380,7 @@ class UpdateMixin(_RestManagerBase):
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
-            dict: The new object data (*not* a RESTObject)
+            The new object data (*not* a RESTObject)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
@@ -392,7 +391,7 @@ class UpdateMixin(_RestManagerBase):
         if id is None:
             path = self.path
         else:
-            path = "%s/%s" % (self.path, id)
+            path = f"{self.path}/{id}"
 
         self._check_missing_update_attrs(new_data)
         files = {}
@@ -434,18 +433,18 @@ class SetMixin(_RestManagerBase):
         """Create or update the object.
 
         Args:
-            key (str): The key of the object to create/update
-            value (str): The value to set for the object
+            key: The key of the object to create/update
+            value: The value to set for the object
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
-            GitlabSetError: If an error occured
+            GitlabSetError: If an error occurred
 
         Returns:
-            obj: The created/updated attribute
+            The created/updated attribute
         """
-        path = "%s/%s" % (self.path, utils.clean_str_id(key))
+        path = f"{self.path}/{utils.clean_str_id(key)}"
         data = {"value": value}
         server_data = self.gitlab.http_put(path, post_data=data, **kwargs)
         if TYPE_CHECKING:
@@ -480,7 +479,7 @@ class DeleteMixin(_RestManagerBase):
         else:
             if not isinstance(id, int):
                 id = utils.clean_str_id(id)
-            path = "%s/%s" % (self.path, id)
+            path = f"{self.path}/{id}"
         self.gitlab.http_delete(path, **kwargs)
 
 
@@ -599,7 +598,7 @@ class UserAgentDetailMixin(_RestObjectBase):
             GitlabAuthenticationError: If authentication is not correct
             GitlabGetError: If the server cannot perform the request
         """
-        path = "%s/%s/user_agent_detail" % (self.manager.path, self.get_id())
+        path = f"{self.manager.path}/{self.get_id()}/user_agent_detail"
         result = self.manager.gitlab.http_get(path, **kwargs)
         if TYPE_CHECKING:
             assert not isinstance(result, requests.Response)
@@ -619,12 +618,12 @@ class AccessRequestMixin(_RestObjectBase):
     )
     @exc.on_http_error(exc.GitlabUpdateError)
     def approve(
-        self, access_level: int = gitlab.DEVELOPER_ACCESS, **kwargs: Any
+        self, access_level: int = gitlab.const.DEVELOPER_ACCESS, **kwargs: Any
     ) -> None:
         """Approve an access request.
 
         Args:
-            access_level (int): The access level for the user
+            access_level: The access level for the user
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -632,7 +631,7 @@ class AccessRequestMixin(_RestObjectBase):
             GitlabUpdateError: If the server fails to perform the request
         """
 
-        path = "%s/%s/approve" % (self.manager.path, self.id)
+        path = f"{self.manager.path}/{self.id}/approve"
         data = {"access_level": access_level}
         server_data = self.manager.gitlab.http_put(path, post_data=data, **kwargs)
         if TYPE_CHECKING:
@@ -655,17 +654,17 @@ class DownloadMixin(_RestObjectBase):
         streamed: bool = False,
         action: Optional[Callable] = None,
         chunk_size: int = 1024,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> Optional[bytes]:
         """Download the archive of a resource export.
 
         Args:
-            streamed (bool): If True the data will be processed by chunks of
+            streamed: If True the data will be processed by chunks of
                 `chunk_size` and each chunk is passed to `action` for
-                reatment
-            action (callable): Callable responsible of dealing with chunk of
+                treatment
+            action: Callable responsible of dealing with chunk of
                 data
-            chunk_size (int): Size of each chunk
+            chunk_size: Size of each chunk
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -673,9 +672,9 @@ class DownloadMixin(_RestObjectBase):
             GitlabGetError: If the server failed to perform the request
 
         Returns:
-            str: The blob content if streamed is False, None otherwise
+            The blob content if streamed is False, None otherwise
         """
-        path = "%s/download" % (self.manager.path)
+        path = f"{self.manager.path}/download"
         result = self.manager.gitlab.http_get(
             path, streamed=streamed, raw=True, **kwargs
         )
@@ -706,7 +705,7 @@ class SubscribableMixin(_RestObjectBase):
             GitlabAuthenticationError: If authentication is not correct
             GitlabSubscribeError: If the subscription cannot be done
         """
-        path = "%s/%s/subscribe" % (self.manager.path, self.get_id())
+        path = f"{self.manager.path}/{self.get_id()}/subscribe"
         server_data = self.manager.gitlab.http_post(path, **kwargs)
         if TYPE_CHECKING:
             assert not isinstance(server_data, requests.Response)
@@ -726,7 +725,7 @@ class SubscribableMixin(_RestObjectBase):
             GitlabAuthenticationError: If authentication is not correct
             GitlabUnsubscribeError: If the unsubscription cannot be done
         """
-        path = "%s/%s/unsubscribe" % (self.manager.path, self.get_id())
+        path = f"{self.manager.path}/{self.get_id()}/unsubscribe"
         server_data = self.manager.gitlab.http_post(path, **kwargs)
         if TYPE_CHECKING:
             assert not isinstance(server_data, requests.Response)
@@ -753,7 +752,7 @@ class TodoMixin(_RestObjectBase):
             GitlabAuthenticationError: If authentication is not correct
             GitlabTodoError: If the todo cannot be set
         """
-        path = "%s/%s/todo" % (self.manager.path, self.get_id())
+        path = f"{self.manager.path}/{self.get_id()}/todo"
         self.manager.gitlab.http_post(path, **kwargs)
 
 
@@ -782,7 +781,7 @@ class TimeTrackingMixin(_RestObjectBase):
         if "time_stats" in self.attributes:
             return self.attributes["time_stats"]
 
-        path = "%s/%s/time_stats" % (self.manager.path, self.get_id())
+        path = f"{self.manager.path}/{self.get_id()}/time_stats"
         result = self.manager.gitlab.http_get(path, **kwargs)
         if TYPE_CHECKING:
             assert not isinstance(result, requests.Response)
@@ -794,14 +793,14 @@ class TimeTrackingMixin(_RestObjectBase):
         """Set an estimated time of work for the object.
 
         Args:
-            duration (str): Duration in human format (e.g. 3h30)
+            duration: Duration in human format (e.g. 3h30)
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
             GitlabTimeTrackingError: If the time tracking update cannot be done
         """
-        path = "%s/%s/time_estimate" % (self.manager.path, self.get_id())
+        path = f"{self.manager.path}/{self.get_id()}/time_estimate"
         data = {"duration": duration}
         result = self.manager.gitlab.http_post(path, post_data=data, **kwargs)
         if TYPE_CHECKING:
@@ -820,7 +819,7 @@ class TimeTrackingMixin(_RestObjectBase):
             GitlabAuthenticationError: If authentication is not correct
             GitlabTimeTrackingError: If the time tracking update cannot be done
         """
-        path = "%s/%s/reset_time_estimate" % (self.manager.path, self.get_id())
+        path = f"{self.manager.path}/{self.get_id()}/reset_time_estimate"
         result = self.manager.gitlab.http_post(path, **kwargs)
         if TYPE_CHECKING:
             assert not isinstance(result, requests.Response)
@@ -832,14 +831,14 @@ class TimeTrackingMixin(_RestObjectBase):
         """Add time spent working on the object.
 
         Args:
-            duration (str): Duration in human format (e.g. 3h30)
+            duration: Duration in human format (e.g. 3h30)
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
             GitlabTimeTrackingError: If the time tracking update cannot be done
         """
-        path = "%s/%s/add_spent_time" % (self.manager.path, self.get_id())
+        path = f"{self.manager.path}/{self.get_id()}/add_spent_time"
         data = {"duration": duration}
         result = self.manager.gitlab.http_post(path, post_data=data, **kwargs)
         if TYPE_CHECKING:
@@ -858,7 +857,7 @@ class TimeTrackingMixin(_RestObjectBase):
             GitlabAuthenticationError: If authentication is not correct
             GitlabTimeTrackingError: If the time tracking update cannot be done
         """
-        path = "%s/%s/reset_spent_time" % (self.manager.path, self.get_id())
+        path = f"{self.manager.path}/{self.get_id()}/reset_spent_time"
         result = self.manager.gitlab.http_post(path, **kwargs)
         if TYPE_CHECKING:
             assert not isinstance(result, requests.Response)
@@ -879,10 +878,10 @@ class ParticipantsMixin(_RestObjectBase):
         """List the participants.
 
         Args:
-            all (bool): If True, return all the items, without pagination
-            per_page (int): Number of items to retrieve per request
-            page (int): ID of the page to return (starts with page 1)
-            as_list (bool): If set to False and no pagination option is
+            all: If True, return all the items, without pagination
+            per_page: Number of items to retrieve per request
+            page: ID of the page to return (starts with page 1)
+            as_list: If set to False and no pagination option is
                 defined, return a generator instead of a list
             **kwargs: Extra options to send to the server (e.g. sudo)
 
@@ -891,10 +890,10 @@ class ParticipantsMixin(_RestObjectBase):
             GitlabListError: If the list could not be retrieved
 
         Returns:
-            RESTObjectList: The list of participants
+            The list of participants
         """
 
-        path = "%s/%s/participants" % (self.manager.path, self.get_id())
+        path = f"{self.manager.path}/{self.get_id()}/participants"
         result = self.manager.gitlab.http_get(path, **kwargs)
         if TYPE_CHECKING:
             assert not isinstance(result, requests.Response)
@@ -910,8 +909,8 @@ class BadgeRenderMixin(_RestManagerBase):
         """Preview link_url and image_url after interpolation.
 
         Args:
-            link_url (str): URL of the badge link
-            image_url (str): URL of the badge image
+            link_url: URL of the badge link
+            image_url: URL of the badge image
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -919,9 +918,9 @@ class BadgeRenderMixin(_RestManagerBase):
             GitlabRenderError: If the rendering failed
 
         Returns:
-            dict: The rendering properties
+            The rendering properties
         """
-        path = "%s/render" % self.path
+        path = f"{self.path}/render"
         data = {"link_url": link_url, "image_url": image_url}
         result = self.gitlab.http_get(path, data, **kwargs)
         if TYPE_CHECKING:
@@ -929,48 +928,48 @@ class BadgeRenderMixin(_RestManagerBase):
         return result
 
 
-class MemberAllMixin(_RestManagerBase):
-    """This mixin is deprecated."""
-
-    _computed_path: Optional[str]
-    _from_parent_attrs: Dict[str, Any]
-    _obj_cls: Optional[Type[base.RESTObject]]
-    _parent: Optional[base.RESTObject]
+class PromoteMixin(_RestObjectBase):
+    _id_attr: Optional[str]
+    _attrs: Dict[str, Any]
+    _module: ModuleType
     _parent_attrs: Dict[str, Any]
-    _path: Optional[str]
-    gitlab: gitlab.Gitlab
+    _updated_attrs: Dict[str, Any]
+    _update_uses_post: bool = False
+    manager: base.RESTManager
 
-    @cli.register_custom_action(("GroupMemberManager", "ProjectMemberManager"))
-    @exc.on_http_error(exc.GitlabListError)
-    def all(self, **kwargs: Any) -> List[base.RESTObject]:
-        """List all the members, included inherited ones.
+    def _get_update_method(
+        self,
+    ) -> Callable[..., Union[Dict[str, Any], requests.Response]]:
+        """Return the HTTP method to use.
 
-        This Method is deprecated.
+        Returns:
+            http_put (default) or http_post
+        """
+        if self._update_uses_post:
+            http_method = self.manager.gitlab.http_post
+        else:
+            http_method = self.manager.gitlab.http_put
+        return http_method
+
+    @exc.on_http_error(exc.GitlabPromoteError)
+    def promote(self, **kwargs: Any) -> Dict[str, Any]:
+        """Promote the item.
 
         Args:
-            all (bool): If True, return all the items, without pagination
-            per_page (int): Number of items to retrieve per request
-            page (int): ID of the page to return (starts with page 1)
-            as_list (bool): If set to False and no pagination option is
-                defined, return a generator instead of a list
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
-            GitlabListError: If the list could not be retrieved
+            GitlabPromoteError: If the item could not be promoted
+            GitlabParsingError: If the json data could not be parsed
 
         Returns:
-            RESTObjectList: The list of members
+            The updated object data (*not* a RESTObject)
         """
 
-        warnings.warn(
-            "The all() method for this object is deprecated "
-            "and will be removed in a future version. Use .members_all.list(all=True), instead.",
-            DeprecationWarning,
-        )
-        path = "%s/all" % self.path
-
+        path = f"{self.manager.path}/{self.id}/promote"
+        http_method = self._get_update_method()
+        result = http_method(path, **kwargs)
         if TYPE_CHECKING:
-            assert self._obj_cls is not None
-        obj = self.gitlab.http_list(path, **kwargs)
-        return [self._obj_cls(self, item) for item in obj]
+            assert not isinstance(result, requests.Response)
+        return result
