@@ -15,15 +15,23 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from urllib.parse import urlparse
+from typing import Any, Callable, Dict, Optional
+from urllib.parse import quote
+
+import requests
 
 
 class _StdoutStream(object):
-    def __call__(self, chunk):
+    def __call__(self, chunk: Any) -> None:
         print(chunk)
 
 
-def response_content(response, streamed, action, chunk_size):
+def response_content(
+    response: requests.Response,
+    streamed: bool,
+    action: Optional[Callable],
+    chunk_size: int,
+) -> Optional[bytes]:
     if streamed is False:
         return response.content
 
@@ -33,25 +41,24 @@ def response_content(response, streamed, action, chunk_size):
     for chunk in response.iter_content(chunk_size=chunk_size):
         if chunk:
             action(chunk)
+    return None
 
 
-def copy_dict(dest, src):
+def copy_dict(dest: Dict[str, Any], src: Dict[str, Any]) -> None:
     for k, v in src.items():
         if isinstance(v, dict):
             # Transform dict values to new attributes. For example:
             # custom_attributes: {'foo', 'bar'} =>
             #   "custom_attributes['foo']": "bar"
             for dict_k, dict_v in v.items():
-                dest["%s[%s]" % (k, dict_k)] = dict_v
+                dest[f"{k}[{dict_k}]"] = dict_v
         else:
             dest[k] = v
 
 
-def clean_str_id(id):
-    return id.replace("/", "%2F").replace("#", "%23")
+def clean_str_id(id: str) -> str:
+    return quote(id, safe="")
 
 
-def sanitized_url(url):
-    parsed = urlparse(url)
-    new_path = parsed.path.replace(".", "%2E")
-    return parsed._replace(path=new_path).geturl()
+def remove_none_from_dict(data: Dict[str, Any]) -> Dict[str, Any]:
+    return {k: v for k, v in data.items() if v is not None}
