@@ -16,126 +16,170 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import print_function
-import inspect
+import argparse
 import operator
 import sys
+from typing import Any, Dict, List, Optional, Type, TYPE_CHECKING, Union
 
 import gitlab
 import gitlab.base
-from gitlab import cli
 import gitlab.v4.objects
+from gitlab import cli
 
 
 class GitlabCLI(object):
-    def __init__(self, gl, what, action, args):
-        self.cls_name = cli.what_to_cls(what)
-        self.cls = gitlab.v4.objects.__dict__[self.cls_name]
+    def __init__(
+        self, gl: gitlab.Gitlab, what: str, action: str, args: Dict[str, str]
+    ) -> None:
+        self.cls: Type[gitlab.base.RESTObject] = cli.what_to_cls(
+            what, namespace=gitlab.v4.objects
+        )
+        self.cls_name = self.cls.__name__
         self.what = what.replace("-", "_")
         self.action = action.lower()
         self.gl = gl
         self.args = args
-        self.mgr_cls = getattr(gitlab.v4.objects, self.cls.__name__ + "Manager")
+        self.mgr_cls: Union[
+            Type[gitlab.mixins.CreateMixin],
+            Type[gitlab.mixins.DeleteMixin],
+            Type[gitlab.mixins.GetMixin],
+            Type[gitlab.mixins.GetWithoutIdMixin],
+            Type[gitlab.mixins.ListMixin],
+            Type[gitlab.mixins.UpdateMixin],
+        ] = getattr(gitlab.v4.objects, f"{self.cls.__name__}Manager")
         # We could do something smart, like splitting the manager name to find
         # parents, build the chain of managers to get to the final object.
         # Instead we do something ugly and efficient: interpolate variables in
         # the class _path attribute, and replace the value with the result.
-        self.mgr_cls._path = self.mgr_cls._path % self.args
+        if TYPE_CHECKING:
+            assert self.mgr_cls._path is not None
+        self.mgr_cls._path = self.mgr_cls._path.format(**self.args)
         self.mgr = self.mgr_cls(gl)
 
-        types = getattr(self.mgr_cls, "_types", {})
-        if types:
-            for attr_name, type_cls in types.items():
+        if self.mgr_cls._types:
+            for attr_name, type_cls in self.mgr_cls._types.items():
                 if attr_name in self.args.keys():
                     obj = type_cls()
                     obj.set_from_cli(self.args[attr_name])
                     self.args[attr_name] = obj.get()
 
-    def __call__(self):
+    def __call__(self) -> Any:
         # Check for a method that matches object + action
-        method = "do_%s_%s" % (self.what, self.action)
+        method = f"do_{self.what}_{self.action}"
         if hasattr(self, method):
             return getattr(self, method)()
 
         # Fallback to standard actions (get, list, create, ...)
-        method = "do_%s" % self.action
+        method = f"do_{self.action}"
         if hasattr(self, method):
             return getattr(self, method)()
 
         # Finally try to find custom methods
         return self.do_custom()
 
-    def do_custom(self):
+    def do_custom(self) -> Any:
         in_obj = cli.custom_actions[self.cls_name][self.action][2]
 
         # Get the object (lazy), then act
         if in_obj:
             data = {}
-            if hasattr(self.mgr, "_from_parent_attrs"):
+            if self.mgr._from_parent_attrs:
                 for k in self.mgr._from_parent_attrs:
                     data[k] = self.args[k]
-            if gitlab.mixins.GetWithoutIdMixin not in inspect.getmro(self.cls):
+            if not issubclass(self.cls, gitlab.mixins.GetWithoutIdMixin):
+                if TYPE_CHECKING:
+                    assert isinstance(self.cls._id_attr, str)
                 data[self.cls._id_attr] = self.args.pop(self.cls._id_attr)
-            o = self.cls(self.mgr, data)
+            obj = self.cls(self.mgr, data)
             method_name = self.action.replace("-", "_")
-            return getattr(o, method_name)(**self.args)
+            return getattr(obj, method_name)(**self.args)
         else:
             return getattr(self.mgr, self.action)(**self.args)
 
-    def do_project_export_download(self):
+    def do_project_export_download(self) -> None:
         try:
             project = self.gl.projects.get(int(self.args["project_id"]), lazy=True)
-            data = project.exports.get().download()
-            if hasattr(sys.stdout, "buffer"):
-                # python3
-                sys.stdout.buffer.write(data)
-            else:
-                sys.stdout.write(data)
+            export_status = project.exports.get()
+            if TYPE_CHECKING:
+                assert export_status is not None
+            data = export_status.download()
+            if TYPE_CHECKING:
+                assert data is not None
+            sys.stdout.buffer.write(data)
 
         except Exception as e:
             cli.die("Impossible to download the export", e)
 
-    def do_create(self):
+    def do_create(self) -> gitlab.base.RESTObject:
+        if TYPE_CHECKING:
+            assert isinstance(self.mgr, gitlab.mixins.CreateMixin)
         try:
-            return self.mgr.create(self.args)
+            result = self.mgr.create(self.args)
         except Exception as e:
             cli.die("Impossible to create object", e)
+        return result
 
-    def do_list(self):
+    def do_list(
+        self,
+    ) -> Union[gitlab.base.RESTObjectList, List[gitlab.base.RESTObject]]:
+        if TYPE_CHECKING:
+            assert isinstance(self.mgr, gitlab.mixins.ListMixin)
         try:
-            return self.mgr.list(**self.args)
+            result = self.mgr.list(**self.args)
         except Exception as e:
             cli.die("Impossible to list objects", e)
+        return result
 
-    def do_get(self):
-        id = None
-        if gitlab.mixins.GetWithoutIdMixin not in inspect.getmro(self.mgr_cls):
-            id = self.args.pop(self.cls._id_attr)
+    def do_get(self) -> Optional[gitlab.base.RESTObject]:
+        if isinstance(self.mgr, gitlab.mixins.GetWithoutIdMixin):
+            try:
+                result = self.mgr.get(id=None, **self.args)
+            except Exception as e:
+                cli.die("Impossible to get object", e)
+            return result
 
+        if TYPE_CHECKING:
+            assert isinstance(self.mgr, gitlab.mixins.GetMixin)
+            assert isinstance(self.cls._id_attr, str)
+
+        id = self.args.pop(self.cls._id_attr)
         try:
-            return self.mgr.get(id, **self.args)
+            result = self.mgr.get(id, lazy=False, **self.args)
         except Exception as e:
             cli.die("Impossible to get object", e)
+        return result
 
-    def do_delete(self):
+    def do_delete(self) -> None:
+        if TYPE_CHECKING:
+            assert isinstance(self.mgr, gitlab.mixins.DeleteMixin)
+            assert isinstance(self.cls._id_attr, str)
         id = self.args.pop(self.cls._id_attr)
         try:
             self.mgr.delete(id, **self.args)
         except Exception as e:
             cli.die("Impossible to destroy object", e)
 
-    def do_update(self):
-        id = None
-        if gitlab.mixins.GetWithoutIdMixin not in inspect.getmro(self.mgr_cls):
+    def do_update(self) -> Dict[str, Any]:
+        if TYPE_CHECKING:
+            assert isinstance(self.mgr, gitlab.mixins.UpdateMixin)
+        if issubclass(self.mgr_cls, gitlab.mixins.GetWithoutIdMixin):
+            id = None
+        else:
+            if TYPE_CHECKING:
+                assert isinstance(self.cls._id_attr, str)
             id = self.args.pop(self.cls._id_attr)
+
         try:
-            return self.mgr.update(id, self.args)
+            result = self.mgr.update(id, self.args)
         except Exception as e:
             cli.die("Impossible to update object", e)
+        return result
 
 
-def _populate_sub_parser_by_class(cls, sub_parser):
-    mgr_cls_name = cls.__name__ + "Manager"
+def _populate_sub_parser_by_class(
+    cls: Type[gitlab.base.RESTObject], sub_parser: argparse._SubParsersAction
+) -> None:
+    mgr_cls_name = f"{cls.__name__}Manager"
     mgr_cls = getattr(gitlab.v4.objects, mgr_cls_name)
 
     for action_name in ["list", "get", "create", "update", "delete"]:
@@ -144,22 +188,17 @@ def _populate_sub_parser_by_class(cls, sub_parser):
 
         sub_parser_action = sub_parser.add_parser(action_name)
         sub_parser_action.add_argument("--sudo", required=False)
-        if hasattr(mgr_cls, "_from_parent_attrs"):
-            [
+        if mgr_cls._from_parent_attrs:
+            for x in mgr_cls._from_parent_attrs:
                 sub_parser_action.add_argument(
-                    "--%s" % x.replace("_", "-"), required=True
+                    f"--{x.replace('_', '-')}", required=True
                 )
-                for x in mgr_cls._from_parent_attrs
-            ]
 
         if action_name == "list":
-            if hasattr(mgr_cls, "_list_filters"):
-                [
-                    sub_parser_action.add_argument(
-                        "--%s" % x.replace("_", "-"), required=False
-                    )
-                    for x in mgr_cls._list_filters
-                ]
+            for x in mgr_cls._list_filters:
+                sub_parser_action.add_argument(
+                    f"--{x.replace('_', '-')}", required=False
+                )
 
             sub_parser_action.add_argument("--page", required=False)
             sub_parser_action.add_argument("--per-page", required=False)
@@ -168,127 +207,101 @@ def _populate_sub_parser_by_class(cls, sub_parser):
         if action_name == "delete":
             if cls._id_attr is not None:
                 id_attr = cls._id_attr.replace("_", "-")
-                sub_parser_action.add_argument("--%s" % id_attr, required=True)
+                sub_parser_action.add_argument(f"--{id_attr}", required=True)
 
         if action_name == "get":
-            if gitlab.mixins.GetWithoutIdMixin not in inspect.getmro(cls):
+            if not issubclass(cls, gitlab.mixins.GetWithoutIdMixin):
                 if cls._id_attr is not None:
                     id_attr = cls._id_attr.replace("_", "-")
-                    sub_parser_action.add_argument("--%s" % id_attr, required=True)
+                    sub_parser_action.add_argument(f"--{id_attr}", required=True)
 
-            if hasattr(mgr_cls, "_optional_get_attrs"):
-                [
-                    sub_parser_action.add_argument(
-                        "--%s" % x.replace("_", "-"), required=False
-                    )
-                    for x in mgr_cls._optional_get_attrs
-                ]
+            for x in mgr_cls._optional_get_attrs:
+                sub_parser_action.add_argument(
+                    f"--{x.replace('_', '-')}", required=False
+                )
 
         if action_name == "create":
-            if hasattr(mgr_cls, "_create_attrs"):
-                [
-                    sub_parser_action.add_argument(
-                        "--%s" % x.replace("_", "-"), required=True
-                    )
-                    for x in mgr_cls._create_attrs[0]
-                ]
-
-                [
-                    sub_parser_action.add_argument(
-                        "--%s" % x.replace("_", "-"), required=False
-                    )
-                    for x in mgr_cls._create_attrs[1]
-                ]
+            for x in mgr_cls._create_attrs.required:
+                sub_parser_action.add_argument(
+                    f"--{x.replace('_', '-')}", required=True
+                )
+            for x in mgr_cls._create_attrs.optional:
+                sub_parser_action.add_argument(
+                    f"--{x.replace('_', '-')}", required=False
+                )
 
         if action_name == "update":
             if cls._id_attr is not None:
                 id_attr = cls._id_attr.replace("_", "-")
-                sub_parser_action.add_argument("--%s" % id_attr, required=True)
+                sub_parser_action.add_argument(f"--{id_attr}", required=True)
 
-            if hasattr(mgr_cls, "_update_attrs"):
-                [
+            for x in mgr_cls._update_attrs.required:
+                if x != cls._id_attr:
                     sub_parser_action.add_argument(
-                        "--%s" % x.replace("_", "-"), required=True
+                        f"--{x.replace('_', '-')}", required=True
                     )
-                    for x in mgr_cls._update_attrs[0]
-                    if x != cls._id_attr
-                ]
 
-                [
+            for x in mgr_cls._update_attrs.optional:
+                if x != cls._id_attr:
                     sub_parser_action.add_argument(
-                        "--%s" % x.replace("_", "-"), required=False
+                        f"--{x.replace('_', '-')}", required=False
                     )
-                    for x in mgr_cls._update_attrs[1]
-                    if x != cls._id_attr
-                ]
 
     if cls.__name__ in cli.custom_actions:
         name = cls.__name__
         for action_name in cli.custom_actions[name]:
             sub_parser_action = sub_parser.add_parser(action_name)
             # Get the attributes for URL/path construction
-            if hasattr(mgr_cls, "_from_parent_attrs"):
-                [
+            if mgr_cls._from_parent_attrs:
+                for x in mgr_cls._from_parent_attrs:
                     sub_parser_action.add_argument(
-                        "--%s" % x.replace("_", "-"), required=True
+                        f"--{x.replace('_', '-')}", required=True
                     )
-                    for x in mgr_cls._from_parent_attrs
-                ]
                 sub_parser_action.add_argument("--sudo", required=False)
 
             # We need to get the object somehow
-            if gitlab.mixins.GetWithoutIdMixin not in inspect.getmro(cls):
+            if not issubclass(cls, gitlab.mixins.GetWithoutIdMixin):
                 if cls._id_attr is not None:
                     id_attr = cls._id_attr.replace("_", "-")
-                    sub_parser_action.add_argument("--%s" % id_attr, required=True)
+                    sub_parser_action.add_argument(f"--{id_attr}", required=True)
 
             required, optional, dummy = cli.custom_actions[name][action_name]
-            [
-                sub_parser_action.add_argument(
-                    "--%s" % x.replace("_", "-"), required=True
-                )
-                for x in required
-                if x != cls._id_attr
-            ]
-            [
-                sub_parser_action.add_argument(
-                    "--%s" % x.replace("_", "-"), required=False
-                )
-                for x in optional
-                if x != cls._id_attr
-            ]
+            for x in required:
+                if x != cls._id_attr:
+                    sub_parser_action.add_argument(
+                        f"--{x.replace('_', '-')}", required=True
+                    )
+            for x in optional:
+                if x != cls._id_attr:
+                    sub_parser_action.add_argument(
+                        f"--{x.replace('_', '-')}", required=False
+                    )
 
     if mgr_cls.__name__ in cli.custom_actions:
         name = mgr_cls.__name__
         for action_name in cli.custom_actions[name]:
             sub_parser_action = sub_parser.add_parser(action_name)
-            if hasattr(mgr_cls, "_from_parent_attrs"):
-                [
+            if mgr_cls._from_parent_attrs:
+                for x in mgr_cls._from_parent_attrs:
                     sub_parser_action.add_argument(
-                        "--%s" % x.replace("_", "-"), required=True
+                        f"--{x.replace('_', '-')}", required=True
                     )
-                    for x in mgr_cls._from_parent_attrs
-                ]
                 sub_parser_action.add_argument("--sudo", required=False)
 
             required, optional, dummy = cli.custom_actions[name][action_name]
-            [
-                sub_parser_action.add_argument(
-                    "--%s" % x.replace("_", "-"), required=True
-                )
-                for x in required
-                if x != cls._id_attr
-            ]
-            [
-                sub_parser_action.add_argument(
-                    "--%s" % x.replace("_", "-"), required=False
-                )
-                for x in optional
-                if x != cls._id_attr
-            ]
+            for x in required:
+                if x != cls._id_attr:
+                    sub_parser_action.add_argument(
+                        f"--{x.replace('_', '-')}", required=True
+                    )
+            for x in optional:
+                if x != cls._id_attr:
+                    sub_parser_action.add_argument(
+                        f"--{x.replace('_', '-')}", required=False
+                    )
 
 
-def extend_parser(parser):
+def extend_parser(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
     subparsers = parser.add_subparsers(
         title="object", dest="what", help="Object to manipulate."
     )
@@ -297,12 +310,11 @@ def extend_parser(parser):
     # populate argparse for all Gitlab Object
     classes = []
     for cls in gitlab.v4.objects.__dict__.values():
-        try:
-            if gitlab.base.RESTManager in inspect.getmro(cls):
-                if cls._obj_cls is not None:
-                    classes.append(cls._obj_cls)
-        except AttributeError:
-            pass
+        if not isinstance(cls, type):
+            continue
+        if issubclass(cls, gitlab.base.RESTManager):
+            if cls._obj_cls is not None:
+                classes.append(cls._obj_cls)
     classes.sort(key=operator.attrgetter("__name__"))
 
     for cls in classes:
@@ -318,7 +330,9 @@ def extend_parser(parser):
     return parser
 
 
-def get_dict(obj, fields):
+def get_dict(
+    obj: Union[str, gitlab.base.RESTObject], fields: List[str]
+) -> Union[str, Dict[str, Any]]:
     if isinstance(obj, str):
         return obj
 
@@ -328,19 +342,24 @@ def get_dict(obj, fields):
 
 
 class JSONPrinter(object):
-    def display(self, d, **kwargs):
+    def display(self, d: Union[str, Dict[str, Any]], **kwargs: Any) -> None:
         import json  # noqa
 
         print(json.dumps(d))
 
-    def display_list(self, data, fields, **kwargs):
+    def display_list(
+        self,
+        data: List[Union[str, gitlab.base.RESTObject]],
+        fields: List[str],
+        **kwargs: Any,
+    ) -> None:
         import json  # noqa
 
         print(json.dumps([get_dict(obj, fields) for obj in data]))
 
 
 class YAMLPrinter(object):
-    def display(self, d, **kwargs):
+    def display(self, d: Union[str, Dict[str, Any]], **kwargs: Any) -> None:
         try:
             import yaml  # noqa
 
@@ -352,7 +371,12 @@ class YAMLPrinter(object):
                 "to use the yaml output feature"
             )
 
-    def display_list(self, data, fields, **kwargs):
+    def display_list(
+        self,
+        data: List[Union[str, gitlab.base.RESTObject]],
+        fields: List[str],
+        **kwargs: Any,
+    ) -> None:
         try:
             import yaml  # noqa
 
@@ -370,20 +394,22 @@ class YAMLPrinter(object):
 
 
 class LegacyPrinter(object):
-    def display(self, d, **kwargs):
+    def display(self, d: Union[str, Dict[str, Any]], **kwargs: Any) -> None:
         verbose = kwargs.get("verbose", False)
         padding = kwargs.get("padding", 0)
-        obj = kwargs.get("obj")
+        obj: Optional[Union[Dict[str, Any], gitlab.base.RESTObject]] = kwargs.get("obj")
+        if TYPE_CHECKING:
+            assert obj is not None
 
-        def display_dict(d, padding):
+        def display_dict(d: Dict[str, Any], padding: int) -> None:
             for k in sorted(d.keys()):
                 v = d[k]
                 if isinstance(v, dict):
-                    print("%s%s:" % (" " * padding, k.replace("_", "-")))
+                    print(f"{' ' * padding}{k.replace('_', '-')}:")
                     new_padding = padding + 2
                     self.display(v, verbose=True, padding=new_padding, obj=v)
                     continue
-                print("%s%s: %s" % (" " * padding, k.replace("_", "-"), v))
+                print(f"{' ' * padding}{k.replace('_', '-')}: {v}")
 
         if verbose:
             if isinstance(obj, dict):
@@ -393,28 +419,35 @@ class LegacyPrinter(object):
             # not a dict, we assume it's a RESTObject
             if obj._id_attr:
                 id = getattr(obj, obj._id_attr, None)
-                print("%s: %s" % (obj._id_attr, id))
+                print(f"{obj._id_attr}: {id}")
             attrs = obj.attributes
             if obj._id_attr:
                 attrs.pop(obj._id_attr)
             display_dict(attrs, padding)
 
         else:
+            if TYPE_CHECKING:
+                assert isinstance(obj, gitlab.base.RESTObject)
             if obj._id_attr:
                 id = getattr(obj, obj._id_attr)
-                print("%s: %s" % (obj._id_attr.replace("_", "-"), id))
-            if hasattr(obj, "_short_print_attr"):
+                print(f"{obj._id_attr.replace('_', '-')}: {id}")
+            if obj._short_print_attr:
                 value = getattr(obj, obj._short_print_attr) or "None"
                 value = value.replace("\r", "").replace("\n", " ")
                 # If the attribute is a note (ProjectCommitComment) then we do
                 # some modifications to fit everything on one line
-                line = "%s: %s" % (obj._short_print_attr, value)
+                line = f"{obj._short_print_attr}: {value}"
                 # ellipsize long lines (comments)
                 if len(line) > 79:
-                    line = line[:76] + "..."
+                    line = f"{line[:76]}..."
                 print(line)
 
-    def display_list(self, data, fields, **kwargs):
+    def display_list(
+        self,
+        data: List[Union[str, gitlab.base.RESTObject]],
+        fields: List[str],
+        **kwargs: Any,
+    ) -> None:
         verbose = kwargs.get("verbose", False)
         for obj in data:
             if isinstance(obj, gitlab.base.RESTObject):
@@ -424,14 +457,28 @@ class LegacyPrinter(object):
             print("")
 
 
-PRINTERS = {"json": JSONPrinter, "legacy": LegacyPrinter, "yaml": YAMLPrinter}
+PRINTERS: Dict[
+    str, Union[Type[JSONPrinter], Type[LegacyPrinter], Type[YAMLPrinter]]
+] = {
+    "json": JSONPrinter,
+    "legacy": LegacyPrinter,
+    "yaml": YAMLPrinter,
+}
 
 
-def run(gl, what, action, args, verbose, output, fields):
-    g_cli = GitlabCLI(gl, what, action, args)
+def run(
+    gl: gitlab.Gitlab,
+    what: str,
+    action: str,
+    args: Dict[str, Any],
+    verbose: bool,
+    output: str,
+    fields: List[str],
+) -> None:
+    g_cli = GitlabCLI(gl=gl, what=what, action=action, args=args)
     data = g_cli()
 
-    printer = PRINTERS[output]()
+    printer: Union[JSONPrinter, LegacyPrinter, YAMLPrinter] = PRINTERS[output]()
 
     if isinstance(data, dict):
         printer.display(data, verbose=True, obj=data)
@@ -441,5 +488,7 @@ def run(gl, what, action, args, verbose, output, fields):
         printer.display(get_dict(data, fields), verbose=verbose, obj=data)
     elif isinstance(data, str):
         print(data)
+    elif isinstance(data, bytes):
+        sys.stdout.buffer.write(data)
     elif hasattr(data, "decode"):
         print(data.decode())
