@@ -1,4 +1,7 @@
 import base64
+from typing import Any, Callable, cast, Dict, List, Optional, TYPE_CHECKING
+
+import requests
 
 from gitlab import cli
 from gitlab import exceptions as exc
@@ -22,23 +25,29 @@ __all__ = [
 class ProjectFile(SaveMixin, ObjectDeleteMixin, RESTObject):
     _id_attr = "file_path"
     _short_print_attr = "file_path"
+    file_path: str
+    manager: "ProjectFileManager"
 
     def decode(self) -> bytes:
         """Returns the decoded content of the file.
 
         Returns:
-            (bytes): the decoded content.
+            The decoded content.
         """
         return base64.b64decode(self.content)
 
-    def save(self, branch, commit_message, **kwargs):
+    # NOTE(jlvillal): Signature doesn't match SaveMixin.save() so ignore
+    # type error
+    def save(  # type: ignore
+        self, branch: str, commit_message: str, **kwargs: Any
+    ) -> None:
         """Save the changes made to the file to the server.
 
         The object is updated to match what the server returns.
 
         Args:
-            branch (str): Branch in which the file will be updated
-            commit_message (str): Message to send with the commit
+            branch: Branch in which the file will be updated
+            commit_message: Message to send with the commit
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -50,12 +59,17 @@ class ProjectFile(SaveMixin, ObjectDeleteMixin, RESTObject):
         self.file_path = self.file_path.replace("/", "%2F")
         super(ProjectFile, self).save(**kwargs)
 
-    def delete(self, branch, commit_message, **kwargs):
+    @exc.on_http_error(exc.GitlabDeleteError)
+    # NOTE(jlvillal): Signature doesn't match DeleteMixin.delete() so ignore
+    # type error
+    def delete(  # type: ignore
+        self, branch: str, commit_message: str, **kwargs: Any
+    ) -> None:
         """Delete the file from the server.
 
         Args:
-            branch (str): Branch from which the file will be removed
-            commit_message (str): Commit message for the deletion
+            branch: Branch from which the file will be removed
+            commit_message: Commit message for the deletion
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -67,7 +81,7 @@ class ProjectFile(SaveMixin, ObjectDeleteMixin, RESTObject):
 
 
 class ProjectFileManager(GetMixin, CreateMixin, UpdateMixin, DeleteMixin, RESTManager):
-    _path = "/projects/%(project_id)s/repository/files"
+    _path = "/projects/{project_id}/repository/files"
     _obj_cls = ProjectFile
     _from_parent_attrs = {"project_id": "id"}
     _create_attrs = RequiredOptional(
@@ -80,12 +94,16 @@ class ProjectFileManager(GetMixin, CreateMixin, UpdateMixin, DeleteMixin, RESTMa
     )
 
     @cli.register_custom_action("ProjectFileManager", ("file_path", "ref"))
-    def get(self, file_path, ref, **kwargs):
+    # NOTE(jlvillal): Signature doesn't match UpdateMixin.update() so ignore
+    # type error
+    def get(  # type: ignore
+        self, file_path: str, ref: str, **kwargs: Any
+    ) -> ProjectFile:
         """Retrieve a single file.
 
         Args:
-            file_path (str): Path of the file to retrieve
-            ref (str): Name of the branch, tag or commit
+            file_path: Path of the file to retrieve
+            ref: Name of the branch, tag or commit
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -93,9 +111,9 @@ class ProjectFileManager(GetMixin, CreateMixin, UpdateMixin, DeleteMixin, RESTMa
             GitlabGetError: If the file could not be retrieved
 
         Returns:
-            object: The generated RESTObject
+            The generated RESTObject
         """
-        return GetMixin.get(self, file_path, ref=ref, **kwargs)
+        return cast(ProjectFile, GetMixin.get(self, file_path, ref=ref, **kwargs))
 
     @cli.register_custom_action(
         "ProjectFileManager",
@@ -103,16 +121,18 @@ class ProjectFileManager(GetMixin, CreateMixin, UpdateMixin, DeleteMixin, RESTMa
         ("encoding", "author_email", "author_name"),
     )
     @exc.on_http_error(exc.GitlabCreateError)
-    def create(self, data, **kwargs):
+    def create(
+        self, data: Optional[Dict[str, Any]] = None, **kwargs: Any
+    ) -> ProjectFile:
         """Create a new object.
 
         Args:
-            data (dict): parameters to send to the server to create the
+            data: parameters to send to the server to create the
                          resource
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
-            RESTObject: a new instance of the managed object class built with
+            a new instance of the managed object class built with
                 the data sent by the server
 
         Raises:
@@ -120,15 +140,23 @@ class ProjectFileManager(GetMixin, CreateMixin, UpdateMixin, DeleteMixin, RESTMa
             GitlabCreateError: If the server cannot perform the request
         """
 
+        if TYPE_CHECKING:
+            assert data is not None
         self._check_missing_create_attrs(data)
         new_data = data.copy()
         file_path = new_data.pop("file_path").replace("/", "%2F")
-        path = "%s/%s" % (self.path, file_path)
+        path = f"{self.path}/{file_path}"
         server_data = self.gitlab.http_post(path, post_data=new_data, **kwargs)
+        if TYPE_CHECKING:
+            assert isinstance(server_data, dict)
         return self._obj_cls(self, server_data)
 
     @exc.on_http_error(exc.GitlabUpdateError)
-    def update(self, file_path, new_data=None, **kwargs):
+    # NOTE(jlvillal): Signature doesn't match UpdateMixin.update() so ignore
+    # type error
+    def update(  # type: ignore
+        self, file_path: str, new_data: Optional[Dict[str, Any]] = None, **kwargs: Any
+    ) -> Dict[str, Any]:
         """Update an object on the server.
 
         Args:
@@ -137,7 +165,7 @@ class ProjectFileManager(GetMixin, CreateMixin, UpdateMixin, DeleteMixin, RESTMa
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
-            dict: The new object data (*not* a RESTObject)
+            The new object data (*not* a RESTObject)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
@@ -147,47 +175,60 @@ class ProjectFileManager(GetMixin, CreateMixin, UpdateMixin, DeleteMixin, RESTMa
         data = new_data.copy()
         file_path = file_path.replace("/", "%2F")
         data["file_path"] = file_path
-        path = "%s/%s" % (self.path, file_path)
+        path = f"{self.path}/{file_path}"
         self._check_missing_update_attrs(data)
-        return self.gitlab.http_put(path, post_data=data, **kwargs)
+        result = self.gitlab.http_put(path, post_data=data, **kwargs)
+        if TYPE_CHECKING:
+            assert isinstance(result, dict)
+        return result
 
     @cli.register_custom_action(
         "ProjectFileManager", ("file_path", "branch", "commit_message")
     )
     @exc.on_http_error(exc.GitlabDeleteError)
-    def delete(self, file_path, branch, commit_message, **kwargs):
+    # NOTE(jlvillal): Signature doesn't match DeleteMixin.delete() so ignore
+    # type error
+    def delete(  # type: ignore
+        self, file_path: str, branch: str, commit_message: str, **kwargs: Any
+    ) -> None:
         """Delete a file on the server.
 
         Args:
-            file_path (str): Path of the file to remove
-            branch (str): Branch from which the file will be removed
-            commit_message (str): Commit message for the deletion
+            file_path: Path of the file to remove
+            branch: Branch from which the file will be removed
+            commit_message: Commit message for the deletion
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
             GitlabDeleteError: If the server cannot perform the request
         """
-        path = "%s/%s" % (self.path, file_path.replace("/", "%2F"))
+        path = f"{self.path}/{file_path.replace('/', '%2F')}"
         data = {"branch": branch, "commit_message": commit_message}
         self.gitlab.http_delete(path, query_data=data, **kwargs)
 
     @cli.register_custom_action("ProjectFileManager", ("file_path", "ref"))
     @exc.on_http_error(exc.GitlabGetError)
     def raw(
-        self, file_path, ref, streamed=False, action=None, chunk_size=1024, **kwargs
-    ):
+        self,
+        file_path: str,
+        ref: str,
+        streamed: bool = False,
+        action: Optional[Callable[..., Any]] = None,
+        chunk_size: int = 1024,
+        **kwargs: Any,
+    ) -> Optional[bytes]:
         """Return the content of a file for a commit.
 
         Args:
-            ref (str): ID of the commit
-            filepath (str): Path of the file to return
-            streamed (bool): If True the data will be processed by chunks of
+            ref: ID of the commit
+            filepath: Path of the file to return
+            streamed: If True the data will be processed by chunks of
                 `chunk_size` and each chunk is passed to `action` for
                 treatment
-            action (callable): Callable responsible of dealing with chunk of
+            action: Callable responsible of dealing with chunk of
                 data
-            chunk_size (int): Size of each chunk
+            chunk_size: Size of each chunk
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -195,24 +236,26 @@ class ProjectFileManager(GetMixin, CreateMixin, UpdateMixin, DeleteMixin, RESTMa
             GitlabGetError: If the file could not be retrieved
 
         Returns:
-            str: The file content
+            The file content
         """
         file_path = file_path.replace("/", "%2F").replace(".", "%2E")
-        path = "%s/%s/raw" % (self.path, file_path)
+        path = f"{self.path}/{file_path}/raw"
         query_data = {"ref": ref}
         result = self.gitlab.http_get(
             path, query_data=query_data, streamed=streamed, raw=True, **kwargs
         )
+        if TYPE_CHECKING:
+            assert isinstance(result, requests.Response)
         return utils.response_content(result, streamed, action, chunk_size)
 
     @cli.register_custom_action("ProjectFileManager", ("file_path", "ref"))
     @exc.on_http_error(exc.GitlabListError)
-    def blame(self, file_path, ref, **kwargs):
+    def blame(self, file_path: str, ref: str, **kwargs: Any) -> List[Dict[str, Any]]:
         """Return the content of a file for a commit.
 
         Args:
-            file_path (str): Path of the file to retrieve
-            ref (str): Name of the branch, tag or commit
+            file_path: Path of the file to retrieve
+            ref: Name of the branch, tag or commit
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -220,9 +263,12 @@ class ProjectFileManager(GetMixin, CreateMixin, UpdateMixin, DeleteMixin, RESTMa
             GitlabListError:  If the server failed to perform the request
 
         Returns:
-            list(blame): a list of commits/lines matching the file
+            A list of commits/lines matching the file
         """
         file_path = file_path.replace("/", "%2F").replace(".", "%2E")
-        path = "%s/%s/blame" % (self.path, file_path)
+        path = f"{self.path}/{file_path}/blame"
         query_data = {"ref": ref}
-        return self.gitlab.http_list(path, query_data, **kwargs)
+        result = self.gitlab.http_list(path, query_data, **kwargs)
+        if TYPE_CHECKING:
+            assert isinstance(result, list)
+        return result
