@@ -1,3 +1,8 @@
+from typing import Any, BinaryIO, cast, Dict, List, Optional, Type, TYPE_CHECKING, Union
+
+import requests
+
+import gitlab
 from gitlab import cli
 from gitlab import exceptions as exc
 from gitlab import types
@@ -43,60 +48,61 @@ __all__ = [
 
 class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
     _short_print_attr = "name"
-    _managers = (
-        ("accessrequests", "GroupAccessRequestManager"),
-        ("audit_events", "GroupAuditEventManager"),
-        ("badges", "GroupBadgeManager"),
-        ("billable_members", "GroupBillableMemberManager"),
-        ("boards", "GroupBoardManager"),
-        ("customattributes", "GroupCustomAttributeManager"),
-        ("descendant_groups", "GroupDescendantGroupManager"),
-        ("exports", "GroupExportManager"),
-        ("epics", "GroupEpicManager"),
-        ("hooks", "GroupHookManager"),
-        ("imports", "GroupImportManager"),
-        ("issues", "GroupIssueManager"),
-        ("issues_statistics", "GroupIssuesStatisticsManager"),
-        ("labels", "GroupLabelManager"),
-        ("members", "GroupMemberManager"),
-        ("members_all", "GroupMemberAllManager"),
-        ("mergerequests", "GroupMergeRequestManager"),
-        ("milestones", "GroupMilestoneManager"),
-        ("notificationsettings", "GroupNotificationSettingsManager"),
-        ("packages", "GroupPackageManager"),
-        ("projects", "GroupProjectManager"),
-        ("runners", "GroupRunnerManager"),
-        ("subgroups", "GroupSubgroupManager"),
-        ("variables", "GroupVariableManager"),
-        ("clusters", "GroupClusterManager"),
-        ("deploytokens", "GroupDeployTokenManager"),
-        ("wikis", "GroupWikiManager"),
-    )
 
-    @cli.register_custom_action("Group", ("to_project_id",))
+    accessrequests: GroupAccessRequestManager
+    audit_events: GroupAuditEventManager
+    badges: GroupBadgeManager
+    billable_members: GroupBillableMemberManager
+    boards: GroupBoardManager
+    clusters: GroupClusterManager
+    customattributes: GroupCustomAttributeManager
+    deploytokens: GroupDeployTokenManager
+    descendant_groups: "GroupDescendantGroupManager"
+    epics: GroupEpicManager
+    exports: GroupExportManager
+    hooks: GroupHookManager
+    imports: GroupImportManager
+    issues: GroupIssueManager
+    issues_statistics: GroupIssuesStatisticsManager
+    labels: GroupLabelManager
+    members: GroupMemberManager
+    members_all: GroupMemberAllManager
+    mergerequests: GroupMergeRequestManager
+    milestones: GroupMilestoneManager
+    notificationsettings: GroupNotificationSettingsManager
+    packages: GroupPackageManager
+    projects: GroupProjectManager
+    runners: GroupRunnerManager
+    subgroups: "GroupSubgroupManager"
+    variables: GroupVariableManager
+    wikis: GroupWikiManager
+
+    @cli.register_custom_action("Group", ("project_id",))
     @exc.on_http_error(exc.GitlabTransferProjectError)
-    def transfer_project(self, to_project_id, **kwargs):
+    def transfer_project(self, project_id: int, **kwargs: Any) -> None:
         """Transfer a project to this group.
 
         Args:
-            to_project_id (int): ID of the project to transfer
+            to_project_id: ID of the project to transfer
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
-            GitlabTransferProjectError: If the project could not be transfered
+            GitlabTransferProjectError: If the project could not be transferred
         """
-        path = "/groups/%s/projects/%s" % (self.id, to_project_id)
+        path = f"/groups/{self.id}/projects/{project_id}"
         self.manager.gitlab.http_post(path, **kwargs)
 
     @cli.register_custom_action("Group", ("scope", "search"))
     @exc.on_http_error(exc.GitlabSearchError)
-    def search(self, scope, search, **kwargs):
+    def search(
+        self, scope: str, search: str, **kwargs: Any
+    ) -> Union[gitlab.GitlabList, List[Dict[str, Any]]]:
         """Search the group resources matching the provided string.'
 
         Args:
-            scope (str): Scope of the search
-            search (str): Search string
+            scope: Scope of the search
+            search: Search string
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -104,55 +110,59 @@ class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
             GitlabSearchError: If the server failed to perform the request
 
         Returns:
-            GitlabList: A list of dicts describing the resources found.
+            A list of dicts describing the resources found.
         """
         data = {"scope": scope, "search": search}
-        path = "/groups/%s/search" % self.get_id()
+        path = f"/groups/{self.get_id()}/search"
         return self.manager.gitlab.http_list(path, query_data=data, **kwargs)
 
     @cli.register_custom_action("Group", ("cn", "group_access", "provider"))
     @exc.on_http_error(exc.GitlabCreateError)
-    def add_ldap_group_link(self, cn, group_access, provider, **kwargs):
+    def add_ldap_group_link(
+        self, cn: str, group_access: int, provider: str, **kwargs: Any
+    ) -> None:
         """Add an LDAP group link.
 
         Args:
-            cn (str): CN of the LDAP group
-            group_access (int): Minimum access level for members of the LDAP
+            cn: CN of the LDAP group
+            group_access: Minimum access level for members of the LDAP
                 group
-            provider (str): LDAP provider for the LDAP group
+            provider: LDAP provider for the LDAP group
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
             GitlabCreateError: If the server cannot perform the request
         """
-        path = "/groups/%s/ldap_group_links" % self.get_id()
+        path = f"/groups/{self.get_id()}/ldap_group_links"
         data = {"cn": cn, "group_access": group_access, "provider": provider}
         self.manager.gitlab.http_post(path, post_data=data, **kwargs)
 
     @cli.register_custom_action("Group", ("cn",), ("provider",))
     @exc.on_http_error(exc.GitlabDeleteError)
-    def delete_ldap_group_link(self, cn, provider=None, **kwargs):
+    def delete_ldap_group_link(
+        self, cn: str, provider: Optional[str] = None, **kwargs: Any
+    ) -> None:
         """Delete an LDAP group link.
 
         Args:
-            cn (str): CN of the LDAP group
-            provider (str): LDAP provider for the LDAP group
+            cn: CN of the LDAP group
+            provider: LDAP provider for the LDAP group
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
             GitlabDeleteError: If the server cannot perform the request
         """
-        path = "/groups/%s/ldap_group_links" % self.get_id()
+        path = f"/groups/{self.get_id()}/ldap_group_links"
         if provider is not None:
-            path += "/%s" % provider
-        path += "/%s" % cn
+            path += f"/{provider}"
+        path += f"/{cn}"
         self.manager.gitlab.http_delete(path)
 
     @cli.register_custom_action("Group")
     @exc.on_http_error(exc.GitlabCreateError)
-    def ldap_sync(self, **kwargs):
+    def ldap_sync(self, **kwargs: Any) -> None:
         """Sync LDAP groups.
 
         Args:
@@ -162,45 +172,57 @@ class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
             GitlabAuthenticationError: If authentication is not correct
             GitlabCreateError: If the server cannot perform the request
         """
-        path = "/groups/%s/ldap_sync" % self.get_id()
+        path = f"/groups/{self.get_id()}/ldap_sync"
         self.manager.gitlab.http_post(path, **kwargs)
 
     @cli.register_custom_action("Group", ("group_id", "group_access"), ("expires_at",))
     @exc.on_http_error(exc.GitlabCreateError)
-    def share(self, group_id, group_access, expires_at=None, **kwargs):
+    def share(
+        self,
+        group_id: int,
+        group_access: int,
+        expires_at: Optional[str] = None,
+        **kwargs: Any,
+    ) -> None:
         """Share the group with a group.
 
         Args:
-            group_id (int): ID of the group.
-            group_access (int): Access level for the group.
+            group_id: ID of the group.
+            group_access: Access level for the group.
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
             GitlabCreateError: If the server failed to perform the request
+
+        Returns:
+            Group
         """
-        path = "/groups/%s/share" % self.get_id()
+        path = f"/groups/{self.get_id()}/share"
         data = {
             "group_id": group_id,
             "group_access": group_access,
             "expires_at": expires_at,
         }
-        self.manager.gitlab.http_post(path, post_data=data, **kwargs)
+        server_data = self.manager.gitlab.http_post(path, post_data=data, **kwargs)
+        if TYPE_CHECKING:
+            assert isinstance(server_data, dict)
+        self._update_attrs(server_data)
 
     @cli.register_custom_action("Group", ("group_id",))
     @exc.on_http_error(exc.GitlabDeleteError)
-    def unshare(self, group_id, **kwargs):
+    def unshare(self, group_id: int, **kwargs: Any) -> None:
         """Delete a shared group link within a group.
 
         Args:
-            group_id (int): ID of the group.
+            group_id: ID of the group.
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
             GitlabDeleteError: If the server failed to perform the request
         """
-        path = "/groups/%s/share/%s" % (self.get_id(), group_id)
+        path = f"/groups/{self.get_id()}/share/{group_id}"
         self.manager.gitlab.http_delete(path, **kwargs)
 
 
@@ -270,15 +292,25 @@ class GroupManager(CRUDMixin, RESTManager):
     )
     _types = {"avatar": types.ImageAttribute, "skip_groups": types.ListAttribute}
 
+    def get(self, id: Union[str, int], lazy: bool = False, **kwargs: Any) -> Group:
+        return cast(Group, super().get(id=id, lazy=lazy, **kwargs))
+
     @exc.on_http_error(exc.GitlabImportError)
-    def import_group(self, file, path, name, parent_id=None, **kwargs):
+    def import_group(
+        self,
+        file: BinaryIO,
+        path: str,
+        name: str,
+        parent_id: Optional[str] = None,
+        **kwargs: Any,
+    ) -> Union[Dict[str, Any], requests.Response]:
         """Import a group from an archive file.
 
         Args:
             file: Data or file object containing the group
-            path (str): The path for the new group to be imported.
-            name (str): The name for the new group.
-            parent_id (str): ID of a parent group that the group will
+            path: The path for the new group to be imported.
+            name: The name for the new group.
+            parent_id: ID of a parent group that the group will
                 be imported into.
             **kwargs: Extra options to send to the server (e.g. sudo)
 
@@ -287,7 +319,7 @@ class GroupManager(CRUDMixin, RESTManager):
             GitlabImportError: If the server failed to perform the request
 
         Returns:
-            dict: A representation of the import status.
+            A representation of the import status.
         """
         files = {"file": ("file.tar.gz", file, "application/octet-stream")}
         data = {"path": path, "name": name}
@@ -304,8 +336,8 @@ class GroupSubgroup(RESTObject):
 
 
 class GroupSubgroupManager(ListMixin, RESTManager):
-    _path = "/groups/%(group_id)s/subgroups"
-    _obj_cls = GroupSubgroup
+    _path = "/groups/{group_id}/subgroups"
+    _obj_cls: Union[Type["GroupDescendantGroup"], Type[GroupSubgroup]] = GroupSubgroup
     _from_parent_attrs = {"group_id": "id"}
     _list_filters = (
         "skip_groups",
@@ -331,5 +363,5 @@ class GroupDescendantGroupManager(GroupSubgroupManager):
     share all attributes with subgroups, except the path and object class.
     """
 
-    _path = "/groups/%(group_id)s/descendant_groups"
-    _obj_cls = GroupDescendantGroup
+    _path = "/groups/{group_id}/descendant_groups"
+    _obj_cls: Type[GroupDescendantGroup] = GroupDescendantGroup
