@@ -1,4 +1,6 @@
-import warnings
+from typing import Any, cast, Dict, Optional, TYPE_CHECKING, Union
+
+import requests
 
 from gitlab import cli
 from gitlab import exceptions as exc
@@ -41,49 +43,20 @@ class ProjectMergeRequestPipeline(RESTObject):
 
 
 class ProjectMergeRequestPipelineManager(CreateMixin, ListMixin, RESTManager):
-    _path = "/projects/%(project_id)s/merge_requests/%(mr_iid)s/pipelines"
+    _path = "/projects/{project_id}/merge_requests/{mr_iid}/pipelines"
     _obj_cls = ProjectMergeRequestPipeline
     _from_parent_attrs = {"project_id": "project_id", "mr_iid": "iid"}
 
-    # If the manager was called directly as a callable via
-    # mr.pipelines(), execute the deprecated method for now.
-    # TODO: in python-gitlab 3.0.0, remove this method entirely.
-
-    @cli.register_custom_action("ProjectMergeRequest", custom_action="pipelines")
-    @exc.on_http_error(exc.GitlabListError)
-    def __call__(self, **kwargs):
-        """List the merge request pipelines.
-
-        Args:
-            **kwargs: Extra options to send to the server (e.g. sudo)
-
-        Raises:
-            GitlabAuthenticationError: If authentication is not correct
-            GitlabListError: If the list could not be retrieved
-
-        Returns:
-            RESTObjectList: List of changes
-        """
-        warnings.warn(
-            "Calling the ProjectMergeRequest.pipelines() method on "
-            "merge request objects directly is deprecated and will be replaced "
-            "by ProjectMergeRequest.pipelines.list() in python-gitlab 3.0.0.\n",
-            DeprecationWarning,
-        )
-        return self.list(**kwargs)
-
 
 class ProjectPipeline(RefreshMixin, ObjectDeleteMixin, RESTObject):
-    _managers = (
-        ("jobs", "ProjectPipelineJobManager"),
-        ("bridges", "ProjectPipelineBridgeManager"),
-        ("variables", "ProjectPipelineVariableManager"),
-        ("test_report", "ProjectPipelineTestReportManager"),
-    )
+    bridges: "ProjectPipelineBridgeManager"
+    jobs: "ProjectPipelineJobManager"
+    test_report: "ProjectPipelineTestReportManager"
+    variables: "ProjectPipelineVariableManager"
 
     @cli.register_custom_action("ProjectPipeline")
     @exc.on_http_error(exc.GitlabPipelineCancelError)
-    def cancel(self, **kwargs):
+    def cancel(self, **kwargs: Any) -> Union[Dict[str, Any], requests.Response]:
         """Cancel the job.
 
         Args:
@@ -93,12 +66,12 @@ class ProjectPipeline(RefreshMixin, ObjectDeleteMixin, RESTObject):
             GitlabAuthenticationError: If authentication is not correct
             GitlabPipelineCancelError: If the request failed
         """
-        path = "%s/%s/cancel" % (self.manager.path, self.get_id())
+        path = f"{self.manager.path}/{self.get_id()}/cancel"
         return self.manager.gitlab.http_post(path)
 
     @cli.register_custom_action("ProjectPipeline")
     @exc.on_http_error(exc.GitlabPipelineRetryError)
-    def retry(self, **kwargs):
+    def retry(self, **kwargs: Any) -> Union[Dict[str, Any], requests.Response]:
         """Retry the job.
 
         Args:
@@ -108,12 +81,12 @@ class ProjectPipeline(RefreshMixin, ObjectDeleteMixin, RESTObject):
             GitlabAuthenticationError: If authentication is not correct
             GitlabPipelineRetryError: If the request failed
         """
-        path = "%s/%s/retry" % (self.manager.path, self.get_id())
+        path = f"{self.manager.path}/{self.get_id()}/retry"
         return self.manager.gitlab.http_post(path)
 
 
 class ProjectPipelineManager(RetrieveMixin, CreateMixin, DeleteMixin, RESTManager):
-    _path = "/projects/%(project_id)s/pipelines"
+    _path = "/projects/{project_id}/pipelines"
     _obj_cls = ProjectPipeline
     _from_parent_attrs = {"project_id": "id"}
     _list_filters = (
@@ -129,11 +102,18 @@ class ProjectPipelineManager(RetrieveMixin, CreateMixin, DeleteMixin, RESTManage
     )
     _create_attrs = RequiredOptional(required=("ref",))
 
-    def create(self, data, **kwargs):
+    def get(
+        self, id: Union[str, int], lazy: bool = False, **kwargs: Any
+    ) -> ProjectPipeline:
+        return cast(ProjectPipeline, super().get(id=id, lazy=lazy, **kwargs))
+
+    def create(
+        self, data: Optional[Dict[str, Any]] = None, **kwargs: Any
+    ) -> ProjectPipeline:
         """Creates a new object.
 
         Args:
-            data (dict): Parameters to send to the server to create the
+            data: Parameters to send to the server to create the
                          resource
             **kwargs: Extra options to send to the server (e.g. sudo)
 
@@ -142,11 +122,15 @@ class ProjectPipelineManager(RetrieveMixin, CreateMixin, DeleteMixin, RESTManage
             GitlabCreateError: If the server cannot perform the request
 
         Returns:
-            RESTObject: A new instance of the managed object class build with
+            A new instance of the managed object class build with
                 the data sent by the server
         """
+        if TYPE_CHECKING:
+            assert self.path is not None
         path = self.path[:-1]  # drop the 's'
-        return CreateMixin.create(self, data, path=path, **kwargs)
+        return cast(
+            ProjectPipeline, CreateMixin.create(self, data, path=path, **kwargs)
+        )
 
 
 class ProjectPipelineJob(RESTObject):
@@ -154,7 +138,7 @@ class ProjectPipelineJob(RESTObject):
 
 
 class ProjectPipelineJobManager(ListMixin, RESTManager):
-    _path = "/projects/%(project_id)s/pipelines/%(pipeline_id)s/jobs"
+    _path = "/projects/{project_id}/pipelines/{pipeline_id}/jobs"
     _obj_cls = ProjectPipelineJob
     _from_parent_attrs = {"project_id": "project_id", "pipeline_id": "id"}
     _list_filters = ("scope", "include_retried")
@@ -165,7 +149,7 @@ class ProjectPipelineBridge(RESTObject):
 
 
 class ProjectPipelineBridgeManager(ListMixin, RESTManager):
-    _path = "/projects/%(project_id)s/pipelines/%(pipeline_id)s/bridges"
+    _path = "/projects/{project_id}/pipelines/{pipeline_id}/bridges"
     _obj_cls = ProjectPipelineBridge
     _from_parent_attrs = {"project_id": "project_id", "pipeline_id": "id"}
     _list_filters = ("scope",)
@@ -176,7 +160,7 @@ class ProjectPipelineVariable(RESTObject):
 
 
 class ProjectPipelineVariableManager(ListMixin, RESTManager):
-    _path = "/projects/%(project_id)s/pipelines/%(pipeline_id)s/variables"
+    _path = "/projects/{project_id}/pipelines/{pipeline_id}/variables"
     _obj_cls = ProjectPipelineVariable
     _from_parent_attrs = {"project_id": "project_id", "pipeline_id": "id"}
 
@@ -188,10 +172,7 @@ class ProjectPipelineScheduleVariable(SaveMixin, ObjectDeleteMixin, RESTObject):
 class ProjectPipelineScheduleVariableManager(
     CreateMixin, UpdateMixin, DeleteMixin, RESTManager
 ):
-    _path = (
-        "/projects/%(project_id)s/pipeline_schedules/"
-        "%(pipeline_schedule_id)s/variables"
-    )
+    _path = "/projects/{project_id}/pipeline_schedules/{pipeline_schedule_id}/variables"
     _obj_cls = ProjectPipelineScheduleVariable
     _from_parent_attrs = {"project_id": "project_id", "pipeline_schedule_id": "id"}
     _create_attrs = RequiredOptional(required=("key", "value"))
@@ -199,11 +180,11 @@ class ProjectPipelineScheduleVariableManager(
 
 
 class ProjectPipelineSchedule(SaveMixin, ObjectDeleteMixin, RESTObject):
-    _managers = (("variables", "ProjectPipelineScheduleVariableManager"),)
+    variables: ProjectPipelineScheduleVariableManager
 
     @cli.register_custom_action("ProjectPipelineSchedule")
     @exc.on_http_error(exc.GitlabOwnershipError)
-    def take_ownership(self, **kwargs):
+    def take_ownership(self, **kwargs: Any) -> None:
         """Update the owner of a pipeline schedule.
 
         Args:
@@ -213,13 +194,15 @@ class ProjectPipelineSchedule(SaveMixin, ObjectDeleteMixin, RESTObject):
             GitlabAuthenticationError: If authentication is not correct
             GitlabOwnershipError: If the request failed
         """
-        path = "%s/%s/take_ownership" % (self.manager.path, self.get_id())
+        path = f"{self.manager.path}/{self.get_id()}/take_ownership"
         server_data = self.manager.gitlab.http_post(path, **kwargs)
+        if TYPE_CHECKING:
+            assert isinstance(server_data, dict)
         self._update_attrs(server_data)
 
     @cli.register_custom_action("ProjectPipelineSchedule")
     @exc.on_http_error(exc.GitlabPipelinePlayError)
-    def play(self, **kwargs):
+    def play(self, **kwargs: Any) -> Dict[str, Any]:
         """Trigger a new scheduled pipeline, which runs immediately.
         The next scheduled run of this pipeline is not affected.
 
@@ -230,14 +213,16 @@ class ProjectPipelineSchedule(SaveMixin, ObjectDeleteMixin, RESTObject):
             GitlabAuthenticationError: If authentication is not correct
             GitlabPipelinePlayError: If the request failed
         """
-        path = "%s/%s/play" % (self.manager.path, self.get_id())
+        path = f"{self.manager.path}/{self.get_id()}/play"
         server_data = self.manager.gitlab.http_post(path, **kwargs)
+        if TYPE_CHECKING:
+            assert isinstance(server_data, dict)
         self._update_attrs(server_data)
         return server_data
 
 
 class ProjectPipelineScheduleManager(CRUDMixin, RESTManager):
-    _path = "/projects/%(project_id)s/pipeline_schedules"
+    _path = "/projects/{project_id}/pipeline_schedules"
     _obj_cls = ProjectPipelineSchedule
     _from_parent_attrs = {"project_id": "id"}
     _create_attrs = RequiredOptional(
@@ -247,12 +232,22 @@ class ProjectPipelineScheduleManager(CRUDMixin, RESTManager):
         optional=("description", "ref", "cron", "cron_timezone", "active"),
     )
 
+    def get(
+        self, id: Union[str, int], lazy: bool = False, **kwargs: Any
+    ) -> ProjectPipelineSchedule:
+        return cast(ProjectPipelineSchedule, super().get(id=id, lazy=lazy, **kwargs))
+
 
 class ProjectPipelineTestReport(RESTObject):
     _id_attr = None
 
 
 class ProjectPipelineTestReportManager(GetWithoutIdMixin, RESTManager):
-    _path = "/projects/%(project_id)s/pipelines/%(pipeline_id)s/test_report"
+    _path = "/projects/{project_id}/pipelines/{pipeline_id}/test_report"
     _obj_cls = ProjectPipelineTestReport
     _from_parent_attrs = {"project_id": "project_id", "pipeline_id": "id"}
+
+    def get(
+        self, id: Optional[Union[int, str]] = None, **kwargs: Any
+    ) -> Optional[ProjectPipelineTestReport]:
+        return cast(Optional[ProjectPipelineTestReport], super().get(id=id, **kwargs))
