@@ -42,6 +42,7 @@ from .merge_request_approvals import (  # noqa: F401
     ProjectApprovalRuleManager,
 )
 from .merge_requests import ProjectMergeRequestManager  # noqa: F401
+from .merge_trains import ProjectMergeTrainManager  # noqa: F401
 from .milestones import ProjectMilestoneManager  # noqa: F401
 from .notes import ProjectNoteManager  # noqa: F401
 from .notification_settings import ProjectNotificationSettingsManager  # noqa: F401
@@ -86,7 +87,7 @@ class GroupProject(RESTObject):
 
 
 class GroupProjectManager(ListMixin, RESTManager):
-    _path = "/groups/%(group_id)s/projects"
+    _path = "/groups/{group_id}/projects"
     _obj_cls = GroupProject
     _from_parent_attrs = {"group_id": "id"}
     _list_filters = (
@@ -108,62 +109,80 @@ class GroupProjectManager(ListMixin, RESTManager):
     )
 
 
+class ProjectGroup(RESTObject):
+    pass
+
+
+class ProjectGroupManager(ListMixin, RESTManager):
+    _path = "/projects/{project_id}/groups"
+    _obj_cls = ProjectGroup
+    _from_parent_attrs = {"project_id": "id"}
+    _list_filters = (
+        "search",
+        "skip_groups",
+        "with_shared",
+        "shared_min_access_level",
+        "shared_visible_only",
+    )
+    _types = {"skip_groups": types.ListAttribute}
+
+
 class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTObject):
     _short_print_attr = "path"
-    _managers = (
-        ("access_tokens", "ProjectAccessTokenManager"),
-        ("accessrequests", "ProjectAccessRequestManager"),
-        ("approvals", "ProjectApprovalManager"),
-        ("approvalrules", "ProjectApprovalRuleManager"),
-        ("badges", "ProjectBadgeManager"),
-        ("boards", "ProjectBoardManager"),
-        ("branches", "ProjectBranchManager"),
-        ("jobs", "ProjectJobManager"),
-        ("commits", "ProjectCommitManager"),
-        ("customattributes", "ProjectCustomAttributeManager"),
-        ("deployments", "ProjectDeploymentManager"),
-        ("environments", "ProjectEnvironmentManager"),
-        ("events", "ProjectEventManager"),
-        ("audit_events", "ProjectAuditEventManager"),
-        ("exports", "ProjectExportManager"),
-        ("files", "ProjectFileManager"),
-        ("forks", "ProjectForkManager"),
-        ("generic_packages", "GenericPackageManager"),
-        ("hooks", "ProjectHookManager"),
-        ("keys", "ProjectKeyManager"),
-        ("imports", "ProjectImportManager"),
-        ("issues", "ProjectIssueManager"),
-        ("labels", "ProjectLabelManager"),
-        ("members", "ProjectMemberManager"),
-        ("members_all", "ProjectMemberAllManager"),
-        ("mergerequests", "ProjectMergeRequestManager"),
-        ("milestones", "ProjectMilestoneManager"),
-        ("notes", "ProjectNoteManager"),
-        ("notificationsettings", "ProjectNotificationSettingsManager"),
-        ("packages", "ProjectPackageManager"),
-        ("pagesdomains", "ProjectPagesDomainManager"),
-        ("pipelines", "ProjectPipelineManager"),
-        ("protectedbranches", "ProjectProtectedBranchManager"),
-        ("protectedtags", "ProjectProtectedTagManager"),
-        ("pipelineschedules", "ProjectPipelineScheduleManager"),
-        ("pushrules", "ProjectPushRulesManager"),
-        ("releases", "ProjectReleaseManager"),
-        ("remote_mirrors", "ProjectRemoteMirrorManager"),
-        ("repositories", "ProjectRegistryRepositoryManager"),
-        ("runners", "ProjectRunnerManager"),
-        ("services", "ProjectServiceManager"),
-        ("snippets", "ProjectSnippetManager"),
-        ("tags", "ProjectTagManager"),
-        ("users", "ProjectUserManager"),
-        ("triggers", "ProjectTriggerManager"),
-        ("variables", "ProjectVariableManager"),
-        ("wikis", "ProjectWikiManager"),
-        ("clusters", "ProjectClusterManager"),
-        ("additionalstatistics", "ProjectAdditionalStatisticsManager"),
-        ("issues_statistics", "ProjectIssuesStatisticsManager"),
-        ("issuesstatistics", "ProjectIssuesStatisticsManager"),  # Deprecated
-        ("deploytokens", "ProjectDeployTokenManager"),
-    )
+
+    access_tokens: ProjectAccessTokenManager
+    accessrequests: ProjectAccessRequestManager
+    additionalstatistics: ProjectAdditionalStatisticsManager
+    approvalrules: ProjectApprovalRuleManager
+    approvals: ProjectApprovalManager
+    audit_events: ProjectAuditEventManager
+    badges: ProjectBadgeManager
+    boards: ProjectBoardManager
+    branches: ProjectBranchManager
+    clusters: ProjectClusterManager
+    commits: ProjectCommitManager
+    customattributes: ProjectCustomAttributeManager
+    deployments: ProjectDeploymentManager
+    deploytokens: ProjectDeployTokenManager
+    environments: ProjectEnvironmentManager
+    events: ProjectEventManager
+    exports: ProjectExportManager
+    files: ProjectFileManager
+    forks: "ProjectForkManager"
+    generic_packages: GenericPackageManager
+    groups: ProjectGroupManager
+    hooks: ProjectHookManager
+    imports: ProjectImportManager
+    issues: ProjectIssueManager
+    issues_statistics: ProjectIssuesStatisticsManager
+    jobs: ProjectJobManager
+    keys: ProjectKeyManager
+    labels: ProjectLabelManager
+    members: ProjectMemberManager
+    members_all: ProjectMemberAllManager
+    mergerequests: ProjectMergeRequestManager
+    merge_trains: ProjectMergeTrainManager
+    milestones: ProjectMilestoneManager
+    notes: ProjectNoteManager
+    notificationsettings: ProjectNotificationSettingsManager
+    packages: ProjectPackageManager
+    pagesdomains: ProjectPagesDomainManager
+    pipelines: ProjectPipelineManager
+    pipelineschedules: ProjectPipelineScheduleManager
+    protectedbranches: ProjectProtectedBranchManager
+    protectedtags: ProjectProtectedTagManager
+    pushrules: ProjectPushRulesManager
+    releases: ProjectReleaseManager
+    remote_mirrors: "ProjectRemoteMirrorManager"
+    repositories: ProjectRegistryRepositoryManager
+    runners: ProjectRunnerManager
+    services: ProjectServiceManager
+    snippets: ProjectSnippetManager
+    tags: ProjectTagManager
+    triggers: ProjectTriggerManager
+    users: ProjectUserManager
+    variables: ProjectVariableManager
+    wikis: ProjectWikiManager
 
     @cli.register_custom_action("Project", ("forked_from_id",))
     @exc.on_http_error(exc.GitlabCreateError)
@@ -171,14 +190,14 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
         """Create a forked from/to relation between existing projects.
 
         Args:
-            forked_from_id (int): The ID of the project that was forked from
+            forked_from_id: The ID of the project that was forked from
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
             GitlabCreateError: If the relation could not be created
         """
-        path = "/projects/%s/fork/%s" % (self.get_id(), forked_from_id)
+        path = f"/projects/{self.get_id()}/fork/{forked_from_id}"
         self.manager.gitlab.http_post(path, **kwargs)
 
     @cli.register_custom_action("Project")
@@ -193,7 +212,7 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             GitlabAuthenticationError: If authentication is not correct
             GitlabDeleteError: If the server failed to perform the request
         """
-        path = "/projects/%s/fork" % self.get_id()
+        path = f"/projects/{self.get_id()}/fork"
         self.manager.gitlab.http_delete(path, **kwargs)
 
     @cli.register_custom_action("Project")
@@ -208,7 +227,7 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             GitlabAuthenticationError: If authentication is not correct
             GitlabGetError: If the server failed to perform the request
         """
-        path = "/projects/%s/languages" % self.get_id()
+        path = f"/projects/{self.get_id()}/languages"
         return self.manager.gitlab.http_get(path, **kwargs)
 
     @cli.register_custom_action("Project")
@@ -223,7 +242,7 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             GitlabAuthenticationError: If authentication is not correct
             GitlabCreateError: If the server failed to perform the request
         """
-        path = "/projects/%s/star" % self.get_id()
+        path = f"/projects/{self.get_id()}/star"
         server_data = self.manager.gitlab.http_post(path, **kwargs)
         if TYPE_CHECKING:
             assert isinstance(server_data, dict)
@@ -241,7 +260,7 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             GitlabAuthenticationError: If authentication is not correct
             GitlabDeleteError: If the server failed to perform the request
         """
-        path = "/projects/%s/unstar" % self.get_id()
+        path = f"/projects/{self.get_id()}/unstar"
         server_data = self.manager.gitlab.http_post(path, **kwargs)
         if TYPE_CHECKING:
             assert isinstance(server_data, dict)
@@ -259,7 +278,7 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             GitlabAuthenticationError: If authentication is not correct
             GitlabCreateError: If the server failed to perform the request
         """
-        path = "/projects/%s/archive" % self.get_id()
+        path = f"/projects/{self.get_id()}/archive"
         server_data = self.manager.gitlab.http_post(path, **kwargs)
         if TYPE_CHECKING:
             assert isinstance(server_data, dict)
@@ -277,7 +296,7 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             GitlabAuthenticationError: If authentication is not correct
             GitlabDeleteError: If the server failed to perform the request
         """
-        path = "/projects/%s/unarchive" % self.get_id()
+        path = f"/projects/{self.get_id()}/unarchive"
         server_data = self.manager.gitlab.http_post(path, **kwargs)
         if TYPE_CHECKING:
             assert isinstance(server_data, dict)
@@ -292,20 +311,20 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
         group_id: int,
         group_access: int,
         expires_at: Optional[str] = None,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> None:
         """Share the project with a group.
 
         Args:
-            group_id (int): ID of the group.
-            group_access (int): Access level for the group.
+            group_id: ID of the group.
+            group_access: Access level for the group.
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
             GitlabCreateError: If the server failed to perform the request
         """
-        path = "/projects/%s/share" % self.get_id()
+        path = f"/projects/{self.get_id()}/share"
         data = {
             "group_id": group_id,
             "group_access": group_access,
@@ -319,14 +338,14 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
         """Delete a shared project link within a group.
 
         Args:
-            group_id (int): ID of the group.
+            group_id: ID of the group.
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
             GitlabDeleteError: If the server failed to perform the request
         """
-        path = "/projects/%s/share/%s" % (self.get_id(), group_id)
+        path = f"/projects/{self.get_id()}/share/{group_id}"
         self.manager.gitlab.http_delete(path, **kwargs)
 
     # variables not supported in CLI
@@ -337,16 +356,16 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
         ref: str,
         token: str,
         variables: Optional[Dict[str, Any]] = None,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> ProjectPipeline:
         """Trigger a CI build.
 
         See https://gitlab.com/help/ci/triggers/README.md#trigger-a-build
 
         Args:
-            ref (str): Commit to build; can be a branch name or a tag
-            token (str): The trigger token
-            variables (dict): Variables passed to the build script
+            ref: Commit to build; can be a branch name or a tag
+            token: The trigger token
+            variables: Variables passed to the build script
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -354,7 +373,7 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             GitlabCreateError: If the server failed to perform the request
         """
         variables = variables or {}
-        path = "/projects/%s/trigger/pipeline" % self.get_id()
+        path = f"/projects/{self.get_id()}/trigger/pipeline"
         post_data = {"ref": ref, "token": token, "variables": variables}
         attrs = self.manager.gitlab.http_post(path, post_data=post_data, **kwargs)
         if TYPE_CHECKING:
@@ -374,7 +393,7 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             GitlabHousekeepingError: If the server failed to perform the
                                      request
         """
-        path = "/projects/%s/housekeeping" % self.get_id()
+        path = f"/projects/{self.get_id()}/housekeeping"
         self.manager.gitlab.http_post(path, **kwargs)
 
     # see #56 - add file attachment features
@@ -385,7 +404,7 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
         filename: str,
         filedata: Optional[bytes] = None,
         filepath: Optional[str] = None,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> Dict[str, Any]:
         """Upload the specified file into the project.
 
@@ -394,9 +413,9 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             Either ``filedata`` or ``filepath`` *MUST* be specified.
 
         Args:
-            filename (str): The name of the file being uploaded
-            filedata (bytes): The raw data of the file being uploaded
-            filepath (str): The path to a local file to upload (optional)
+            filename: The name of the file being uploaded
+            filedata: The raw data of the file being uploaded
+            filepath: The path to a local file to upload (optional)
 
         Raises:
             GitlabConnectionError: If the server cannot be reached
@@ -407,7 +426,7 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
                 specified
 
         Returns:
-            dict: A ``dict`` with the keys:
+            A ``dict`` with the keys:
                 * ``alt`` - The alternate text for the upload
                 * ``url`` - The direct url to the uploaded file
                 * ``markdown`` - Markdown for the uploaded file
@@ -422,7 +441,7 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             with open(filepath, "rb") as f:
                 filedata = f.read()
 
-        url = "/projects/%(id)s/uploads" % {"id": self.id}
+        url = f"/projects/{self.id}/uploads"
         file_info = {"file": (filename, filedata)}
         data = self.manager.gitlab.http_post(url, files=file_info)
 
@@ -438,18 +457,18 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
         streamed: bool = False,
         action: Optional[Callable] = None,
         chunk_size: int = 1024,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> Optional[bytes]:
         """Return a snapshot of the repository.
 
         Args:
-            wiki (bool): If True return the wiki repository
-            streamed (bool): If True the data will be processed by chunks of
+            wiki: If True return the wiki repository
+            streamed: If True the data will be processed by chunks of
                 `chunk_size` and each chunk is passed to `action` for
                 treatment.
-            action (callable): Callable responsible of dealing with chunk of
+            action: Callable responsible of dealing with chunk of
                 data
-            chunk_size (int): Size of each chunk
+            chunk_size: Size of each chunk
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -457,9 +476,9 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             GitlabGetError: If the content could not be retrieved
 
         Returns:
-            str: The uncompressed tar archive of the repository
+            The uncompressed tar archive of the repository
         """
-        path = "/projects/%s/snapshot" % self.get_id()
+        path = f"/projects/{self.get_id()}/snapshot"
         result = self.manager.gitlab.http_get(
             path, streamed=streamed, raw=True, **kwargs
         )
@@ -475,8 +494,8 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
         """Search the project resources matching the provided string.'
 
         Args:
-            scope (str): Scope of the search
-            search (str): Search string
+            scope: Scope of the search
+            search: Search string
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -484,10 +503,10 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             GitlabSearchError: If the server failed to perform the request
 
         Returns:
-            GitlabList: A list of dicts describing the resources found.
+            A list of dicts describing the resources found.
         """
         data = {"scope": scope, "search": search}
-        path = "/projects/%s/search" % self.get_id()
+        path = f"/projects/{self.get_id()}/search"
         return self.manager.gitlab.http_list(path, query_data=data, **kwargs)
 
     @cli.register_custom_action("Project")
@@ -502,7 +521,7 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             GitlabAuthenticationError: If authentication is not correct
             GitlabCreateError: If the server failed to perform the request
         """
-        path = "/projects/%s/mirror/pull" % self.get_id()
+        path = f"/projects/{self.get_id()}/mirror/pull"
         self.manager.gitlab.http_post(path, **kwargs)
 
     @cli.register_custom_action("Project", ("to_namespace",))
@@ -511,15 +530,15 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
         """Transfer a project to the given namespace ID
 
         Args:
-            to_namespace (str): ID or path of the namespace to transfer the
+            to_namespace: ID or path of the namespace to transfer the
             project to
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
-            GitlabTransferProjectError: If the project could not be transfered
+            GitlabTransferProjectError: If the project could not be transferred
         """
-        path = "/projects/%s/transfer" % (self.id,)
+        path = f"/projects/{self.id}/transfer"
         self.manager.gitlab.http_put(
             path, post_data={"namespace": to_namespace}, **kwargs
         )
@@ -533,22 +552,22 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
         streamed: bool = False,
         action: Optional[Callable] = None,
         chunk_size: int = 1024,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> Optional[bytes]:
         """Get the job artifacts archive from a specific tag or branch.
 
         Args:
-            ref_name (str): Branch or tag name in repository. HEAD or SHA references
+            ref_name: Branch or tag name in repository. HEAD or SHA references
             are not supported.
-            artifact_path (str): Path to a file inside the artifacts archive.
-            job (str): The name of the job.
-            job_token (str): Job token for multi-project pipeline triggers.
-            streamed (bool): If True the data will be processed by chunks of
+            artifact_path: Path to a file inside the artifacts archive.
+            job: The name of the job.
+            job_token: Job token for multi-project pipeline triggers.
+            streamed: If True the data will be processed by chunks of
                 `chunk_size` and each chunk is passed to `action` for
                 treatment
-            action (callable): Callable responsible of dealing with chunk of
+            action: Callable responsible of dealing with chunk of
                 data
-            chunk_size (int): Size of each chunk
+            chunk_size: Size of each chunk
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -556,9 +575,9 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             GitlabGetError: If the artifacts could not be retrieved
 
         Returns:
-            str: The artifacts if `streamed` is False, None otherwise.
+            The artifacts if `streamed` is False, None otherwise.
         """
-        path = "/projects/%s/jobs/artifacts/%s/download" % (self.get_id(), ref_name)
+        path = f"/projects/{self.get_id()}/jobs/artifacts/{ref_name}/download"
         result = self.manager.gitlab.http_get(
             path, job=job, streamed=streamed, raw=True, **kwargs
         )
@@ -576,20 +595,22 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
         streamed: bool = False,
         action: Optional[Callable] = None,
         chunk_size: int = 1024,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> Optional[bytes]:
-        """Download a single artifact file from a specific tag or branch from within the job’s artifacts archive.
+        """Download a single artifact file from a specific tag or branch from
+        within the job’s artifacts archive.
 
         Args:
-            ref_name (str): Branch or tag name in repository. HEAD or SHA references are not supported.
-            artifact_path (str): Path to a file inside the artifacts archive.
-            job (str): The name of the job.
-            streamed (bool): If True the data will be processed by chunks of
+            ref_name: Branch or tag name in repository. HEAD or SHA references
+                are not supported.
+            artifact_path: Path to a file inside the artifacts archive.
+            job: The name of the job.
+            streamed: If True the data will be processed by chunks of
                 `chunk_size` and each chunk is passed to `action` for
                 treatment
-            action (callable): Callable responsible of dealing with chunk of
+            action: Callable responsible of dealing with chunk of
                 data
-            chunk_size (int): Size of each chunk
+            chunk_size: Size of each chunk
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -597,14 +618,12 @@ class Project(RefreshMixin, SaveMixin, ObjectDeleteMixin, RepositoryMixin, RESTO
             GitlabGetError: If the artifacts could not be retrieved
 
         Returns:
-            str: The artifacts if `streamed` is False, None otherwise.
+            The artifacts if `streamed` is False, None otherwise.
         """
 
-        path = "/projects/%s/jobs/artifacts/%s/raw/%s?job=%s" % (
-            self.get_id(),
-            ref_name,
-            artifact_path,
-            job,
+        path = (
+            f"/projects/{self.get_id()}/jobs/artifacts/{ref_name}/raw/"
+            f"{artifact_path}?job={job}"
         )
         result = self.manager.gitlab.http_get(
             path, streamed=streamed, raw=True, **kwargs
@@ -673,6 +692,7 @@ class ProjectManager(CRUDMixin, RESTManager):
             "show_default_award_emojis",
             "snippets_access_level",
             "snippets_enabled",
+            "squash_option",
             "tag_list",
             "template_name",
             "template_project_id",
@@ -741,6 +761,7 @@ class ProjectManager(CRUDMixin, RESTManager):
             "show_default_award_emojis",
             "snippets_access_level",
             "snippets_enabled",
+            "squash_option",
             "suggestion_commit_message",
             "tag_list",
             "visibility",
@@ -789,18 +810,18 @@ class ProjectManager(CRUDMixin, RESTManager):
         namespace: Optional[str] = None,
         overwrite: bool = False,
         override_params: Optional[Dict[str, Any]] = None,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> Union[Dict[str, Any], requests.Response]:
         """Import a project from an archive file.
 
         Args:
             file: Data or file object containing the project
-            path (str): Name and path for the new project
-            namespace (str): The ID or path of the namespace that the project
+            path: Name and path for the new project
+            namespace: The ID or path of the namespace that the project
                 will be imported to
-            overwrite (bool): If True overwrite an existing project with the
+            overwrite: If True overwrite an existing project with the
                 same path
-            override_params (dict): Set the specific settings for the project
+            override_params: Set the specific settings for the project
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -808,13 +829,13 @@ class ProjectManager(CRUDMixin, RESTManager):
             GitlabListError: If the server failed to perform the request
 
         Returns:
-            dict: A representation of the import status.
+            A representation of the import status.
         """
         files = {"file": ("file.tar.gz", file, "application/octet-stream")}
         data = {"path": path, "overwrite": str(overwrite)}
         if override_params:
             for k, v in override_params.items():
-                data["override_params[%s]" % k] = v
+                data[f"override_params[{k}]"] = v
         if name is not None:
             data["name"] = name
         if namespace:
@@ -832,7 +853,7 @@ class ProjectManager(CRUDMixin, RESTManager):
         bitbucket_server_repo: str,
         new_name: Optional[str] = None,
         target_namespace: Optional[str] = None,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> Union[Dict[str, Any], requests.Response]:
         """Import a project from BitBucket Server to Gitlab (schedule the import)
 
@@ -843,18 +864,19 @@ class ProjectManager(CRUDMixin, RESTManager):
 
         .. note::
             This request may take longer than most other API requests.
-            So this method will specify a 60 second default timeout if none is specified.
+            So this method will specify a 60 second default timeout if none is
+            specified.
             A timeout can be specified via kwargs to override this functionality.
 
         Args:
-            bitbucket_server_url (str): Bitbucket Server URL
-            bitbucket_server_username (str): Bitbucket Server Username
-            personal_access_token (str): Bitbucket Server personal access
+            bitbucket_server_url: Bitbucket Server URL
+            bitbucket_server_username: Bitbucket Server Username
+            personal_access_token: Bitbucket Server personal access
                 token/password
-            bitbucket_server_project (str): Bitbucket Project Key
-            bitbucket_server_repo (str): Bitbucket Repository Name
-            new_name (str): New repository name (Optional)
-            target_namespace (str): Namespace to import repository into.
+            bitbucket_server_project: Bitbucket Project Key
+            bitbucket_server_repo: Bitbucket Repository Name
+            new_name: New repository name (Optional)
+            target_namespace: Namespace to import repository into.
                 Supports subgroups like /namespace/subgroup (Optional)
             **kwargs: Extra options to send to the server (e.g. sudo)
 
@@ -863,7 +885,7 @@ class ProjectManager(CRUDMixin, RESTManager):
             GitlabListError: If the server failed to perform the request
 
         Returns:
-            dict: A representation of the import status.
+            A representation of the import status.
 
         Example:
 
@@ -920,7 +942,7 @@ class ProjectManager(CRUDMixin, RESTManager):
         repo_id: int,
         target_namespace: str,
         new_name: Optional[str] = None,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> Union[Dict[str, Any], requests.Response]:
         """Import a project from Github to Gitlab (schedule the import)
 
@@ -931,14 +953,15 @@ class ProjectManager(CRUDMixin, RESTManager):
 
         .. note::
             This request may take longer than most other API requests.
-            So this method will specify a 60 second default timeout if none is specified.
+            So this method will specify a 60 second default timeout if none is
+            specified.
             A timeout can be specified via kwargs to override this functionality.
 
         Args:
-            personal_access_token (str): GitHub personal access token
-            repo_id (int): Github repository ID
-            target_namespace (str): Namespace to import repo into
-            new_name (str): New repo name (Optional)
+            personal_access_token: GitHub personal access token
+            repo_id: Github repository ID
+            target_namespace: Namespace to import repo into
+            new_name: New repo name (Optional)
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
@@ -946,7 +969,7 @@ class ProjectManager(CRUDMixin, RESTManager):
             GitlabListError: If the server failed to perform the request
 
         Returns:
-            dict: A representation of the import status.
+            A representation of the import status.
 
         Example:
 
@@ -991,7 +1014,7 @@ class ProjectFork(RESTObject):
 
 
 class ProjectForkManager(CreateMixin, ListMixin, RESTManager):
-    _path = "/projects/%(project_id)s/forks"
+    _path = "/projects/{project_id}/forks"
     _obj_cls = ProjectFork
     _from_parent_attrs = {"project_id": "id"}
     _list_filters = (
@@ -1017,7 +1040,7 @@ class ProjectForkManager(CreateMixin, ListMixin, RESTManager):
         """Creates a new object.
 
         Args:
-            data (dict): Parameters to send to the server to create the
+            data: Parameters to send to the server to create the
                          resource
             **kwargs: Extra options to send to the server (e.g. sudo)
 
@@ -1026,7 +1049,7 @@ class ProjectForkManager(CreateMixin, ListMixin, RESTManager):
             GitlabCreateError: If the server cannot perform the request
 
         Returns:
-            RESTObject: A new instance of the managed object class build with
+            A new instance of the managed object class build with
                 the data sent by the server
         """
         if TYPE_CHECKING:
@@ -1040,7 +1063,7 @@ class ProjectRemoteMirror(SaveMixin, RESTObject):
 
 
 class ProjectRemoteMirrorManager(ListMixin, CreateMixin, UpdateMixin, RESTManager):
-    _path = "/projects/%(project_id)s/remote_mirrors"
+    _path = "/projects/{project_id}/remote_mirrors"
     _obj_cls = ProjectRemoteMirror
     _from_parent_attrs = {"project_id": "id"}
     _create_attrs = RequiredOptional(
