@@ -1,7 +1,16 @@
+"""
+GitLab API:
+https://docs.gitlab.com/ee/api/users.html
+https://docs.gitlab.com/ee/api/projects.html#list-projects-starred-by-a-user
+"""
+from typing import Any, cast, Dict, List, Optional, Union
+
+import requests
+
 from gitlab import cli
 from gitlab import exceptions as exc
 from gitlab import types
-from gitlab.base import RequiredOptional, RESTManager, RESTObject
+from gitlab.base import RequiredOptional, RESTManager, RESTObject, RESTObjectList
 from gitlab.mixins import (
     CreateMixin,
     CRUDMixin,
@@ -17,6 +26,7 @@ from gitlab.mixins import (
 
 from .custom_attributes import UserCustomAttributeManager  # noqa: F401
 from .events import UserEventManager  # noqa: F401
+from .personal_access_tokens import UserPersonalAccessTokenManager  # noqa: F401
 
 __all__ = [
     "CurrentUserEmail",
@@ -33,6 +43,8 @@ __all__ = [
     "UserManager",
     "ProjectUser",
     "ProjectUserManager",
+    "StarredProject",
+    "StarredProjectManager",
     "UserEmail",
     "UserEmailManager",
     "UserActivities",
@@ -62,6 +74,11 @@ class CurrentUserEmailManager(RetrieveMixin, CreateMixin, DeleteMixin, RESTManag
     _obj_cls = CurrentUserEmail
     _create_attrs = RequiredOptional(required=("email",))
 
+    def get(
+        self, id: Union[str, int], lazy: bool = False, **kwargs: Any
+    ) -> CurrentUserEmail:
+        return cast(CurrentUserEmail, super().get(id=id, lazy=lazy, **kwargs))
+
 
 class CurrentUserGPGKey(ObjectDeleteMixin, RESTObject):
     pass
@@ -72,6 +89,11 @@ class CurrentUserGPGKeyManager(RetrieveMixin, CreateMixin, DeleteMixin, RESTMana
     _obj_cls = CurrentUserGPGKey
     _create_attrs = RequiredOptional(required=("key",))
 
+    def get(
+        self, id: Union[str, int], lazy: bool = False, **kwargs: Any
+    ) -> CurrentUserGPGKey:
+        return cast(CurrentUserGPGKey, super().get(id=id, lazy=lazy, **kwargs))
+
 
 class CurrentUserKey(ObjectDeleteMixin, RESTObject):
     _short_print_attr = "title"
@@ -82,6 +104,11 @@ class CurrentUserKeyManager(RetrieveMixin, CreateMixin, DeleteMixin, RESTManager
     _obj_cls = CurrentUserKey
     _create_attrs = RequiredOptional(required=("title", "key"))
 
+    def get(
+        self, id: Union[str, int], lazy: bool = False, **kwargs: Any
+    ) -> CurrentUserKey:
+        return cast(CurrentUserKey, super().get(id=id, lazy=lazy, **kwargs))
+
 
 class CurrentUserStatus(SaveMixin, RESTObject):
     _id_attr = None
@@ -93,43 +120,53 @@ class CurrentUserStatusManager(GetWithoutIdMixin, UpdateMixin, RESTManager):
     _obj_cls = CurrentUserStatus
     _update_attrs = RequiredOptional(optional=("emoji", "message"))
 
+    def get(
+        self, id: Optional[Union[int, str]] = None, **kwargs: Any
+    ) -> Optional[CurrentUserStatus]:
+        return cast(Optional[CurrentUserStatus], super().get(id=id, **kwargs))
+
 
 class CurrentUser(RESTObject):
     _id_attr = None
     _short_print_attr = "username"
-    _managers = (
-        ("status", "CurrentUserStatusManager"),
-        ("emails", "CurrentUserEmailManager"),
-        ("gpgkeys", "CurrentUserGPGKeyManager"),
-        ("keys", "CurrentUserKeyManager"),
-    )
+
+    emails: CurrentUserEmailManager
+    gpgkeys: CurrentUserGPGKeyManager
+    keys: CurrentUserKeyManager
+    status: CurrentUserStatusManager
 
 
 class CurrentUserManager(GetWithoutIdMixin, RESTManager):
     _path = "/user"
     _obj_cls = CurrentUser
 
+    def get(
+        self, id: Optional[Union[int, str]] = None, **kwargs: Any
+    ) -> Optional[CurrentUser]:
+        return cast(Optional[CurrentUser], super().get(id=id, **kwargs))
+
 
 class User(SaveMixin, ObjectDeleteMixin, RESTObject):
     _short_print_attr = "username"
-    _managers = (
-        ("customattributes", "UserCustomAttributeManager"),
-        ("emails", "UserEmailManager"),
-        ("followers_users", "UserFollowersManager"),
-        ("following_users", "UserFollowingManager"),
-        ("events", "UserEventManager"),
-        ("gpgkeys", "UserGPGKeyManager"),
-        ("identityproviders", "UserIdentityProviderManager"),
-        ("impersonationtokens", "UserImpersonationTokenManager"),
-        ("keys", "UserKeyManager"),
-        ("memberships", "UserMembershipManager"),
-        ("projects", "UserProjectManager"),
-        ("status", "UserStatusManager"),
-    )
+
+    customattributes: UserCustomAttributeManager
+    emails: "UserEmailManager"
+    events: UserEventManager
+    followers_users: "UserFollowersManager"
+    following_users: "UserFollowingManager"
+    gpgkeys: "UserGPGKeyManager"
+    identityproviders: "UserIdentityProviderManager"
+    impersonationtokens: "UserImpersonationTokenManager"
+    keys: "UserKeyManager"
+    memberships: "UserMembershipManager"
+    personal_access_tokens: UserPersonalAccessTokenManager
+    projects: "UserProjectManager"
+    starred_projects: "StarredProjectManager"
+    status: "UserStatusManager"
 
     @cli.register_custom_action("User")
     @exc.on_http_error(exc.GitlabBlockError)
-    def block(self, **kwargs):
+    def block(self, **kwargs: Any) -> Union[Dict[str, Any], requests.Response]:
         """Block the user.
 
         Args:
@@ -140,9 +177,9 @@ class User(SaveMixin, ObjectDeleteMixin, RESTObject):
             GitlabBlockError: If the user could not be blocked
 
         Returns:
-            bool: Whether the user status has been changed
+            Whether the user status has been changed
         """
-        path = "/users/%s/block" % self.id
+        path = f"/users/{self.id}/block"
         server_data = self.manager.gitlab.http_post(path, **kwargs)
         if server_data is True:
             self._attrs["state"] = "blocked"
@@ -150,7 +187,7 @@ class User(SaveMixin, ObjectDeleteMixin, RESTObject):
 
     @cli.register_custom_action("User")
     @exc.on_http_error(exc.GitlabFollowError)
-    def follow(self, **kwargs):
+    def follow(self, **kwargs: Any) -> Union[Dict[str, Any], requests.Response]:
         """Follow the user.
 
         Args:
@@ -161,14 +198,14 @@ class User(SaveMixin, ObjectDeleteMixin, RESTObject):
             GitlabFollowError: If the user could not be followed
 
         Returns:
-            dict: The new object data (*not* a RESTObject)
+            The new object data (*not* a RESTObject)
         """
-        path = "/users/%s/follow" % self.id
+        path = f"/users/{self.id}/follow"
         return self.manager.gitlab.http_post(path, **kwargs)
 
     @cli.register_custom_action("User")
     @exc.on_http_error(exc.GitlabUnfollowError)
-    def unfollow(self, **kwargs):
+    def unfollow(self, **kwargs: Any) -> Union[Dict[str, Any], requests.Response]:
         """Unfollow the user.
 
         Args:
@@ -179,14 +216,14 @@ class User(SaveMixin, ObjectDeleteMixin, RESTObject):
             GitlabUnfollowError: If the user could not be followed
 
         Returns:
-            dict: The new object data (*not* a RESTObject)
+            The new object data (*not* a RESTObject)
         """
-        path = "/users/%s/unfollow" % self.id
+        path = f"/users/{self.id}/unfollow"
         return self.manager.gitlab.http_post(path, **kwargs)
 
     @cli.register_custom_action("User")
     @exc.on_http_error(exc.GitlabUnblockError)
-    def unblock(self, **kwargs):
+    def unblock(self, **kwargs: Any) -> Union[Dict[str, Any], requests.Response]:
         """Unblock the user.
 
         Args:
@@ -197,9 +234,9 @@ class User(SaveMixin, ObjectDeleteMixin, RESTObject):
             GitlabUnblockError: If the user could not be unblocked
 
         Returns:
-            bool: Whether the user status has been changed
+            Whether the user status has been changed
         """
-        path = "/users/%s/unblock" % self.id
+        path = f"/users/{self.id}/unblock"
         server_data = self.manager.gitlab.http_post(path, **kwargs)
         if server_data is True:
             self._attrs["state"] = "active"
@@ -207,7 +244,7 @@ class User(SaveMixin, ObjectDeleteMixin, RESTObject):
 
     @cli.register_custom_action("User")
     @exc.on_http_error(exc.GitlabDeactivateError)
-    def deactivate(self, **kwargs):
+    def deactivate(self, **kwargs: Any) -> Union[Dict[str, Any], requests.Response]:
         """Deactivate the user.
 
         Args:
@@ -218,9 +255,9 @@ class User(SaveMixin, ObjectDeleteMixin, RESTObject):
             GitlabDeactivateError: If the user could not be deactivated
 
         Returns:
-            bool: Whether the user status has been changed
+            Whether the user status has been changed
         """
-        path = "/users/%s/deactivate" % self.id
+        path = f"/users/{self.id}/deactivate"
         server_data = self.manager.gitlab.http_post(path, **kwargs)
         if server_data:
             self._attrs["state"] = "deactivated"
@@ -228,7 +265,7 @@ class User(SaveMixin, ObjectDeleteMixin, RESTObject):
 
     @cli.register_custom_action("User")
     @exc.on_http_error(exc.GitlabActivateError)
-    def activate(self, **kwargs):
+    def activate(self, **kwargs: Any) -> Union[Dict[str, Any], requests.Response]:
         """Activate the user.
 
         Args:
@@ -239,9 +276,9 @@ class User(SaveMixin, ObjectDeleteMixin, RESTObject):
             GitlabActivateError: If the user could not be activated
 
         Returns:
-            bool: Whether the user status has been changed
+            Whether the user status has been changed
         """
-        path = "/users/%s/activate" % self.id
+        path = f"/users/{self.id}/activate"
         server_data = self.manager.gitlab.http_post(path, **kwargs)
         if server_data:
             self._attrs["state"] = "active"
@@ -319,13 +356,16 @@ class UserManager(CRUDMixin, RESTManager):
     )
     _types = {"confirm": types.LowercaseStringAttribute, "avatar": types.ImageAttribute}
 
+    def get(self, id: Union[str, int], lazy: bool = False, **kwargs: Any) -> User:
+        return cast(User, super().get(id=id, lazy=lazy, **kwargs))
+
 
 class ProjectUser(RESTObject):
     pass
 
 
 class ProjectUserManager(ListMixin, RESTManager):
-    _path = "/projects/%(project_id)s/users"
+    _path = "/projects/{project_id}/users"
     _obj_cls = ProjectUser
     _from_parent_attrs = {"project_id": "id"}
     _list_filters = ("search", "skip_users")
@@ -337,11 +377,14 @@ class UserEmail(ObjectDeleteMixin, RESTObject):
 
 
 class UserEmailManager(RetrieveMixin, CreateMixin, DeleteMixin, RESTManager):
-    _path = "/users/%(user_id)s/emails"
+    _path = "/users/{user_id}/emails"
     _obj_cls = UserEmail
     _from_parent_attrs = {"user_id": "id"}
     _create_attrs = RequiredOptional(required=("email",))
 
+    def get(self, id: Union[str, int], lazy: bool = False, **kwargs: Any) -> UserEmail:
+        return cast(UserEmail, super().get(id=id, lazy=lazy, **kwargs))
+
 
 class UserActivities(RESTObject):
     _id_attr = "username"
@@ -353,10 +396,15 @@ class UserStatus(RESTObject):
 
 
 class UserStatusManager(GetWithoutIdMixin, RESTManager):
-    _path = "/users/%(user_id)s/status"
+    _path = "/users/{user_id}/status"
     _obj_cls = UserStatus
     _from_parent_attrs = {"user_id": "id"}
 
+    def get(
+        self, id: Optional[Union[int, str]] = None, **kwargs: Any
+    ) -> Optional[UserStatus]:
+        return cast(Optional[UserStatus], super().get(id=id, **kwargs))
+
 
 class UserActivitiesManager(ListMixin, RESTManager):
     _path = "/user/activities"
@@ -368,18 +416,21 @@ class UserGPGKey(ObjectDeleteMixin, RESTObject):
 
 
 class UserGPGKeyManager(RetrieveMixin, CreateMixin, DeleteMixin, RESTManager):
-    _path = "/users/%(user_id)s/gpg_keys"
+    _path = "/users/{user_id}/gpg_keys"
     _obj_cls = UserGPGKey
     _from_parent_attrs = {"user_id": "id"}
     _create_attrs = RequiredOptional(required=("key",))
 
+    def get(self, id: Union[str, int], lazy: bool = False, **kwargs: Any) -> UserGPGKey:
+        return cast(UserGPGKey, super().get(id=id, lazy=lazy, **kwargs))
+
 
 class UserKey(ObjectDeleteMixin, RESTObject):
     pass
 
 
 class UserKeyManager(ListMixin, CreateMixin, DeleteMixin, RESTManager):
-    _path = "/users/%(user_id)s/keys"
+    _path = "/users/{user_id}/keys"
     _obj_cls = UserKey
     _from_parent_attrs = {"user_id": "id"}
     _create_attrs = RequiredOptional(required=("title", "key"))
@@ -392,7 +443,7 @@ class UserIdentityProviderManager(DeleteMixin, RESTManager):
     functionality for deletion of user identities by provider.
     """
 
-    _path = "/users/%(user_id)s/identities"
+    _path = "/users/{user_id}/identities"
     _from_parent_attrs = {"user_id": "id"}
 
 
@@ -401,7 +452,7 @@ class UserImpersonationToken(ObjectDeleteMixin, RESTObject):
 
 
 class UserImpersonationTokenManager(NoUpdateMixin, RESTManager):
-    _path = "/users/%(user_id)s/impersonation_tokens"
+    _path = "/users/{user_id}/impersonation_tokens"
     _obj_cls = UserImpersonationToken
     _from_parent_attrs = {"user_id": "id"}
     _create_attrs = RequiredOptional(
@@ -409,17 +460,27 @@ class UserImpersonationTokenManager(NoUpdateMixin, RESTManager):
     )
     _list_filters = ("state",)
 
+    def get(
+        self, id: Union[str, int], lazy: bool = False, **kwargs: Any
+    ) -> UserImpersonationToken:
+        return cast(UserImpersonationToken, super().get(id=id, lazy=lazy, **kwargs))
+
 
 class UserMembership(RESTObject):
     _id_attr = "source_id"
 
 
 class UserMembershipManager(RetrieveMixin, RESTManager):
-    _path = "/users/%(user_id)s/memberships"
+    _path = "/users/{user_id}/memberships"
     _obj_cls = UserMembership
     _from_parent_attrs = {"user_id": "id"}
     _list_filters = ("type",)
 
+    def get(
+        self, id: Union[str, int], lazy: bool = False, **kwargs: Any
+    ) -> UserMembership:
+        return cast(UserMembership, super().get(id=id, lazy=lazy, **kwargs))
+
 
 # Having this outside projects avoids circular imports due to ProjectUser
 class UserProject(RESTObject):
@@ -427,7 +488,7 @@ class UserProject(RESTObject):
 
 
 class UserProjectManager(ListMixin, CreateMixin, RESTManager):
-    _path = "/projects/user/%(user_id)s"
+    _path = "/projects/user/{user_id}"
     _obj_cls = UserProject
     _from_parent_attrs = {"user_id": "id"}
     _create_attrs = RequiredOptional(
@@ -439,6 +500,7 @@ class UserProjectManager(ListMixin, CreateMixin, RESTManager):
             "merge_requests_enabled",
             "wiki_enabled",
             "snippets_enabled",
+            "squash_option",
             "public",
             "visibility",
             "description",
@@ -470,38 +532,64 @@ class UserProjectManager(ListMixin, CreateMixin, RESTManager):
         "id_before",
     )
 
-    def list(self, **kwargs):
+    def list(self, **kwargs: Any) -> Union[RESTObjectList, List[RESTObject]]:
         """Retrieve a list of objects.
 
         Args:
-            all (bool): If True, return all the items, without pagination
-            per_page (int): Number of items to retrieve per request
-            page (int): ID of the page to return (starts with page 1)
-            as_list (bool): If set to False and no pagination option is
+            all: If True, return all the items, without pagination
+            per_page: Number of items to retrieve per request
+            page: ID of the page to return (starts with page 1)
+            as_list: If set to False and no pagination option is
                 defined, return a generator instead of a list
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
-            list: The list of objects, or a generator if `as_list` is False
+            The list of objects, or a generator if `as_list` is False
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
             GitlabListError: If the server cannot perform the request
         """
         if self._parent:
-            path = "/users/%s/projects" % self._parent.id
+            path = f"/users/{self._parent.id}/projects"
         else:
-            path = "/users/%s/projects" % kwargs["user_id"]
+            path = f"/users/{kwargs['user_id']}/projects"
         return ListMixin.list(self, path=path, **kwargs)
 
 
+class StarredProject(RESTObject):
+    pass
+
+
+class StarredProjectManager(ListMixin, RESTManager):
+    _path = "/users/{user_id}/starred_projects"
+    _obj_cls = StarredProject
+    _from_parent_attrs = {"user_id": "id"}
+    _list_filters = (
+        "archived",
+        "membership",
+        "min_access_level",
+        "order_by",
+        "owned",
+        "search",
+        "simple",
+        "sort",
+        "starred",
+        "statistics",
+        "visibility",
+        "with_custom_attributes",
+        "with_issues_enabled",
+        "with_merge_requests_enabled",
+    )
+
+
 class UserFollowersManager(ListMixin, RESTManager):
-    _path = "/users/%(user_id)s/followers"
+    _path = "/users/{user_id}/followers"
     _obj_cls = User
     _from_parent_attrs = {"user_id": "id"}
 
 
 class UserFollowingManager(ListMixin, RESTManager):
-    _path = "/users/%(user_id)s/following"
+    _path = "/users/{user_id}/following"
     _obj_cls = User
     _from_parent_attrs = {"user_id": "id"}
