@@ -36,7 +36,11 @@ def test_groups(gl):
 
     filtered_groups = gl.groups.list(skip_groups=[group3.id, group4.id])
     assert group3 not in filtered_groups
+    assert group4 not in filtered_groups
+
+    filtered_groups = gl.groups.list(skip_groups=[group3.id])
     assert group3 not in filtered_groups
+    assert group4 in filtered_groups
 
     group1.members.create(
         {"access_level": gitlab.const.OWNER_ACCESS, "user_id": user.id}
@@ -89,7 +93,6 @@ def test_groups(gl):
 
     group1.members.delete(user.id)
     assert len(group1.members.list()) == 2
-    assert len(group1.members.all())  # Deprecated
     assert len(group1.members_all.list())
     member = group1.members.get(user2.id)
     member.access_level = gitlab.const.OWNER_ACCESS
