@@ -1,3 +1,5 @@
+import uuid
+
 import pytest
 
 import gitlab
@@ -106,7 +108,7 @@ def test_project_file_uploads(project):
     uploaded_file = project.upload(filename, file_contents)
     assert uploaded_file["alt"] == filename
     assert uploaded_file["url"].startswith("/uploads/")
-    assert uploaded_file["url"].endswith("/" + filename)
+    assert uploaded_file["url"].endswith(f"/{filename}")
     assert uploaded_file["markdown"] == "[{}]({})".format(
         uploaded_file["alt"], uploaded_file["url"]
     )
@@ -159,6 +161,28 @@ def test_project_labels(project):
     assert len(project.labels.list()) == 0
 
 
+def test_project_label_promotion(gl, group):
+    """
+    Label promotion requires the project to be a child of a group (not in a user namespace)
+
+    """
+    _id = uuid.uuid4().hex
+    data = {
+        "name": f"test-project-{_id}",
+        "namespace_id": group.id,
+    }
+    project = gl.projects.create(data)
+
+    label_name = "promoteme"
+    promoted_label = project.labels.create({"name": label_name, "color": "#112233"})
+    promoted_label.promote()
+
+    assert any(label.name == label_name for label in group.labels.list())
+
+    group.labels.delete(label_name)
+    assert not any(label.name == label_name for label in group.labels.list())
+
+
 def test_project_milestones(project):
     milestone = project.milestones.create({"title": "milestone1"})
     assert len(project.milestones.list()) == 1
@@ -175,6 +199,27 @@ def test_project_milestones(project):
     assert len(milestone.merge_requests()) == 0
 
 
+def test_project_milestone_promotion(gl, group):
+    """
+    Milestone promotion requires the project to be a child of a group (not in a user namespace)
+
+    """
+    _id = uuid.uuid4().hex
+    data = {
+        "name": f"test-project-{_id}",
+        "namespace_id": group.id,
+    }
+    project = gl.projects.create(data)
+
+    milestone_title = "promoteme"
+    promoted_milestone = project.milestones.create({"title": milestone_title})
+    promoted_milestone.promote()
+
+    assert any(
+        milestone.title == milestone_title for milestone in group.milestones.list()
+    )
+
+
 def test_project_pages_domains(gl, project):
     domain = project.pagesdomains.create({"domain": "foo.domain.com"})
     assert len(project.pagesdomains.list()) == 1
@@ -240,13 +285,9 @@ def test_project_stars(project):
 
 
 def test_project_tags(project, project_file):
-    tag = project.tags.create({"tag_name": "v1.0", "ref": "master"})
+    tag = project.tags.create({"tag_name": "v1.0", "ref": "main"})
     assert len(project.tags.list()) == 1
 
-    tag.set_release_description("Description 1")
-    tag.set_release_description("Description 2")
-    assert tag.release["description"] == "Description 2"
-
     tag.delete()
     assert len(project.tags.list()) == 0
 
@@ -270,3 +311,21 @@ def test_project_wiki(project):
     wiki.save()
     wiki.delete()
     assert len(project.wikis.list()) == 0
+
+
+def test_project_groups_list(gl, group):
+    """Test listing groups of a project"""
+    # Create a subgroup of our top-group, we will place our new project inside
+    # this group.
+    group2 = gl.groups.create(
+        {"name": "group2_proj", "path": "group2_proj", "parent_id": group.id}
+    )
+    data = {
+        "name": "test-project-tpsg",
+        "namespace_id": group2.id,
+    }
+    project = gl.projects.create(data)
+
+    groups = project.groups.list()
+    group_ids = set([x.id for x in groups])
+    assert set((group.id, group2.id)) == group_ids
